//
//  Mcapi.cs - class encapsulization for API function library
//
//  $Id: Mcapi.cs 856 2015-01-05 17:55:13Z brian $
//
//  Copyright (c) 2006-2015 by Precision Micro Control Corp. All rights reserved.
//
using System;
using System.Text;
using System.Runtime.InteropServices;

namespace McapiNet
{
	/// <summary>
	/// The Mcapi class provides access to the classic function library of the Motion Control API.
	/// </summary>
	public class Mcapi
	{
		/// <summary>
		/// Mcapi class constructor.
		/// </summary>
		public Mcapi()
		{
			m_hCtlr = 0;
			m_throw_exceptions = false;
		}

		//
		// -- Data -------------------------------------------------------------
		//
		private Int16 m_hCtlr;
		private Boolean m_throw_exceptions;
#if MCAPI_ISX64
        private const string mcapidll = "mcapi64.dll";          // 64-bit build uses 64-bit MCAPI
#else
        private const string mcapidll = "mcapi32.dll";          // 32-bit build uses 32-bit MCAPI
#endif

        //
		// -- Constant declarations --------------------------------------------
		//

		/// <summary>Apply command to all axes (wildcard). Not applicable to all functions.</summary>
		public const Int16 AllAxes = 0;
		/// <summary>Maximum valid controller ID value.</summary>
		public const Int16 MaxId = 15;

		//#define MC_MODULE_TYPE     0x000f
		//#define MC_MODULE_SUBTYPE  0x00f0

		/// <summary>Flags for BlockBegin() mode parameter.</summary>
		public enum BlockType : int
		{
			/// <summary>Compound command.</summary>
			Compound = 0,
			/// <summary>Generate background task.</summary>
			Task = 1,
			/// <summary>Create a macro command.</summary>
			Macro = 2,
			/// <summary>Reset macro storage.</summary>
			ResetMacros = 3,
			/// <summary>Cancel block.</summary>
			Cancel = 4,
			/// <summary>User mode contour segment.</summary>
			User = 5,
			/// <summary>Linear contour segment.</summary>
			Linear = 6,
			/// <summary>Clockwise arc contour segment.</summary>
			Clockwise = 7,
			/// <summary>Counter-clockwise arc contour segment.</summary>
			CounterClockwise = 8
		}

		/// <summary>Flags for GetCaptureData() and CaptureModes member of AxisConfiguration structure.</summary>
		[Flags()]
			public enum CaptureType : int
		{
			/// <summary>Actual position data.</summary>
			Actual = 16,
			/// <summary>Position error (difference between optimal and actual position).</summary>
			Error = 32,
			/// <summary>Optimal (ideal) position data.</summary>
			Optimal = 64,
			/// <summary>Torque output.</summary>
			Torque = 128,
			/// <summary>Axis supports advanced capture features (delay and period parameters).</summary>
			Advanced = 256,
			/// <summary>Auxiliary encoder position data.</summary>
			Auxiliary = 512,
			/// <summary>Status word data.</summary>
			Status = 1024
		}

		/// <summary>Flags for ConfigureCompare().</summary>
		[Flags()]
			public enum CompareType : int
		{
			/// <summary>Disable compare.</summary>
			Disable = 0,
			/// <summary>Enable compare.</summary>
			Enable = 1,
			/// <summary>Static mode.</summary>
			Static = 1,
			/// <summary>Toggle ouput mode.</summary>
			Toggle = 2,
			/// <summary>One shot mode.</summary>
			OneShot = 3,
            /// <summary>Strobe mode.</summary>
            Strobe = 5,
            /// <summary>Invert active level of output (may be combined with other flags).</summary>
			Invert = 0x0080
		}

		/// <summary>Flags for ControllerType member of ParamEx structure.</summary>
		[Flags()]
			public enum ControllerType : int
		{
			/// <summary>No controller present.</summary>
			None = 0,
			/// <summary>DCX-PC100.</summary>
			DCXPC100 = 1,
			/// <summary>DCX-AT200.</summary>
			DCXAT200 = 3,
			/// <summary>DC2-PC (all models).</summary>
			DC2PC100 = 4,
			/// <summary>DC2-STN (standalone).</summary>
			DC2STN = 5,
			/// <summary>DCX-AT300.</summary>
			DCXAT300 = 6,
			/// <summary>DCX-PCI300.</summary>
			DCXPCI300 = 7,
			/// <summary>DCX-PCI100.</summary>
			DCXPCI100 = 8,
			/// <summary>MultFlex PCI (all models).</summary>
			MFXPCI1000 = 9,
			/// <summary>MultFlex ETH (all models).</summary>
			MFXETH1000 = 10
		}

		/// <summary>Flags for GetCount().</summary>
		[Flags()]
			public enum CountType : int
		{
			/// <summary>Number of capture points.</summary>
			Cature = 1,
			/// <summary>Number of compare points.</summary>
			Compare = 2,
			/// <summary>Number of executed contouring commands.</summary>
			Contour = 4,
			/// <summary>Number of output filter coefficients.</summary>
			Filter = 8,
			/// <summary>Maximum number of output filter coefficients supported.</summary>
			FilterMax = 16,
			/// <summary>Current position recording index.</summary>
			RecordIdx = 16
		}

		/// <summary>Flags for functions taht take a variable data type.</summary>
		public enum DataType : int
		{
			/// <summary>No argument.</summary>
			None = 0,
			/// <summary>Argument in in a controller register.</summary>
			Register = 1,
			/// <summary>Argument is a 32-bit integer.</summary>
			Long = 2,
			/// <summary>Argument is a 32-bit floating point value.</summary>
			Float = 3,
			/// <summary>Argument is a 64-bit floating point value.</summary>
			Double = 4,
			/// <summary>Argument is a null-terminated string or string buffer.</summary>
			String = 5
		}

		/// <summary>Flags for ConfigureDigitalIO(), GetDigitalIOConfig().</summary>
		[Flags()]
			public enum DigitalIOType : short
		{
			/// <summary>Configure(d) as input.</summary>
			Input = 1,
			/// <summary>Configure(d) as output.</summary>
			Output = 2,
			/// <summary>Configure(d) for high true logic.</summary>
			High = 4,
			/// <summary>Configure(d) for low true logic.</summary>
			Low = 8,
			/// <summary>Configure(d) as latching input.</summary>
			Latch = 16,
			/// <summary>I/O channel type is fixed (not configurable).</summary>
			Fixed = 256,
			/// <summary>I/O channel supports latching.</summary>
			Latchable = 512,
			/// <summary>I/O channel is configured as an auxiliary stepper.</summary>
			Stepper = 1024
		}

		/// <summary>Flags for EnableEncoderFault().</summary>
		[Flags()]
			public enum Encoder : int
		{
			/// <summary>Select no encoder (turn fault detection off).</summary>
			None = 0,
			/// <summary>Select primary encoder.</summary>
			Primary = 1,
			/// <summary>Select auxiliary encoder.</summary>
			Auxiliary = 2,
			/// <summary>Select both primary and auxiliary encoders.</summary>
			Both = Primary | Auxiliary
		}

		/// <summary>Error codes.</summary>
		public enum Error : int
		{
			/// <summary>No error.</summary>
			NoError = 0,
			/// <summary>no controller assigned for this ID.</summary>
			NoController = 1,
			/// <summary>Mcapi is out of handles.</summary>
			OutOfHandles = 2,
			/// <summary>Cannot open - controller already open in exclusive mode.</summary>
			OpenExclusive = 3,
			/// <summary>Cannot open - controller already open in different mode (legacy controllers).</summary>
			ModeUnavailable = 4,
			/// <summary>Controller doesn't support this mode.</summary>
			UnsupportedMode = 5,
			/// <summary>Couldn't initialize the device driver.</summary>
			InitializeDriver = 6,
			/// <summary>Controller hardware not present.</summary>
			NotPresent = 7,
			/// <summary>Mcapi memory allocation error.</summary>
			AllocateMemory = 8,
			/// <summary>Windows function reported an error.</summary>
			WindowsError = 9,
            /// <summary>OPerating sytem function reported an error (alias of WindowsError).</summary>
            OSError = 9,
            /// <summary>Controller doesn't support this feature.</summary>
			NotSupported = 11,
			/// <summary>Function is obsolete.</summary>
			Obsolete = 12,
			/// <summary>Invalid controller handle.</summary>
			ControllerHandle = 13,
			/// <summary>Invalid window handle.</summary>
			WindowHandle = 14,
			/// <summary>Axis number out of range.</summary>
			AxisNumber = 15,
			/// <summary>Axis type doesn't support this feature.</summary>
			AxisType = 16,
			/// <summary>Cannot use "Mcapi.AllAxes" for this function.</summary>
			AllAxes = 17,
			/// <summary>Parameter was out of range.</summary>
			Range = 18,
			/// <summary>Constant value inappropriate.</summary>
			Constant = 19,
			/// <summary>Unexpected or unknown reply.</summary>
			UnknownReply = 20,
			/// <summary>Controller failed to reply.</summary>
			NoReply = 21,
			/// <summary>Reply size incorrect.</summary>
			ReplySize = 22,
			/// <summary>Wrong axis for reply.</summary>
			ReplyAxis = 23,
			/// <summary>Reply is for different command.</summary>
			ReplyCommand = 24,
			/// <summary>Controller failed to respond.</summary>
			Timeout = 25,
			/// <summary>Block mode error.</summary>
			BlockMode = 26,
			/// <summary>Communications port (RS232) error.</summary>
			SerialPort = 27,
			/// <summary>Operation was canceled.</summary>
			Cancel = 28,
			/// <summary>Restore operation could not find data.</summary>
			NotFound = 29,
			/// <summary>Feature not initialized.</summary>
			NotInitialized = 30,
			/// <summary>Axis was enabled or moving.</summary>
			AxisActive = 31,
            /// <summary>Socket error.</summary>
            Socket = 32
        }

		/// <summary>Flags for Get/SetModuleInputMode() mode parameter.</summary>
		public enum InputMode : uint
		{
			/// <summary>Stepper axis open loop mode.</summary>
			OpenLoop = 0,
			/// <summary>Stepper axis closed loop mode.</summary>
			ClosedLoop = 1
		}

		/// <summary>Flags for IntegralOption member of FilterEx structure.</summary>
		public enum IntegralGainMode : int
		{
			/// <summary>Normal, always on operation.</summary>
			Normal = 0,
			/// <summary>Freeze integral gain while moving, re-enable when move is complete.</summary>
			Freeze = 1,
			/// <summary>Freeze and zero integral gain while moving, re-enable when move is complete.</summary>
			Zero = 2
		}

		/// <summary>Flags for LearnPoint().</summary>
		public enum Learn : short
		{
			/// <summary>Learn current actual position.</summary>
			Position = 1,
			/// <summary>Learn current desired position.</summary>
			Target = 2
		}			

		/// <summary>Flags for Get/SetLimits() and limit members of MotionEx struccutre.</summary>
		[Flags()]
			public enum LimitMode : int
		{
			/// <summary>Select turn motor off stopping mode.</summary>
			Off = 0,
			/// <summary>Select positive limit.</summary>
			Plus = 1,
			/// <summary>Select negative limit.</summary>
			Minus = 2,
			/// <summary>Select both positive and negative limits.</summary>
			Both = Plus | Minus,
			/// <summary>Select abrupt stopping mode.</summary>
			Abrupt = 4,
			/// <summary>Select smooth stopping mode.</summary>
			Smooth = 8,
			/// <summary>Invert limit polarity.</summary>
			Invert = 0x0080
		}

		/// <summary>Flags for ModuleType member of AxisConfig structure.</summary>
		public enum ModuleType : int
		{
			/// <summary>Series 100 basic servo.</summary>
			MC100 = 5,
			/// <summary>Series 100 direct motor drive basic servo.</summary>
			MC110 = 4,
			/// <summary>Series 100 basic stepper.</summary>
			MC150 = 6,
			/// <summary>Series 100 basic stepper with encoder.</summary>
			MC160 = 7,
			/// <summary>Series 200 advanced servo.</summary>
			MC200 = 0,
			/// <summary>Series 200 direct motor drive advanced servo.</summary>
			MC210 = 16,
			/// <summary>Series 200 advanced stepper.</summary>
			MC260 = 1,
			/// <summary>Series 300 dsp-based advanced servo.</summary>
			MC300 = 2,
			/// <summary>Series 300 dsp-based dual servo.</summary>
			MC302 = 22,
			/// <summary>Series 300 dsp-based commutating servo.</summary>
			MC320 = 162,
			/// <summary>Series 300 dsp-based advanced stepper.</summary>
			MC360 = 3,
			/// <summary>Series 300 dsp-based dual servo.</summary>
			MC362 = 23,
			/// <summary>Digital I/O module.</summary>
			MC400 = 8,
			/// <summary>Analog I/O module.</summary>
			MC500 = 12,
			/// <summary>RS232 communications module.</summary>
			MF300 = 10,
			/// <summary>IEEE-488 (GPIB) module.</summary>
			MF310 = 9,
			/// <summary>No module present.</summary>
			NO_MODULE = 15,
			/// <summary>Multiflex advanced servo.</summary>
			MFXSERVO = 252,
			/// <summary>Multiflex advanced stepper.</summary>
			MFXSTEPPER = 253,
			/// <summary>DC2 basic servo.</summary>
			DC2SERVO = 254,
			/// <summary>DC2 basic stepper.</summary>
			DC2STEPPER = 255
		}

		/// <summary>Flags for AXISCONFIG MotorType member.</summary>
		public enum MotorType : int
		{
			/// <summary>Axis is a servo motor.</summary>
			Servo = 1,
			/// <summary>Axis is a stepper motor.</summary>
			Stepper = 2
		}

		/// <summary>Flags for Direction() and Direction member of MotionEx structure.</summary>
		public enum MoveDirection : int
		{
			/// <summary>Positive direction (position increasing).</summary>
			Positive = 1,
			/// <summary>Negative direction (position decreasing).</summary>
			Negative = 2
		}

		/// <summary>Flags for Open() mode parameter.</summary>
		[Flags()]
			public enum OpenMode : ushort
		{ 
			/// <summary>Open motion controller handle in ASCII mode.</summary>
			Ascii = 0x0001, 
			/// <summary>Open motion controller handle in BINARY mode.</summary>
			Binary = 0x0002, 
			/// <summary>Combine with Ascii or Binary to request exclusive access to this motion controller (do not allow other calls to open).</summary>
			Exclusive = 0x8000,
			/// <summary>Ascii and Exclusive flags combined.</summary>
			AsciiExclusive = Ascii | Exclusive, 
			/// <summary>Binary and Exclusive flags combined.</summary>
			BinaryExclusive = Binary | Exclusive
		};

		/// <summary>Flags for Get/SetOperatingMode() mode parameter.</summary>
		public enum OperatingMode : int
		{
			/// <summary>Axis mode contouring.</summary>
			Contour = 0,
			/// <summary>Axis mode gain.</summary>
			Gain = 1,
			/// <summary>Axis mode position.</summary>
			Position = 2,
			/// <summary>Axis mode torque.</summary>
			Torque = 3,
			/// <summary>Axis mode velocity.</summary>
			Velocity = 4,
			/// <summary>Axis mode unknown.</summary>
			Unknown = 5
		}

		/// <summary>Flags for Get/SetModuleOutputMode() mode parameter.</summary>
		public enum OutputMode : int
		{
			/// <summary>Bipolar servo.</summary>
			Bipolar = 0,
			/// <summary>Unipolar servo.</summary>
			Unipolar = 1,
			/// <summary>Pulse and direction mode stepper.</summary>
			PulseAndDirection = 0,
			/// <summary>Step clockwise/step counter-clockwise stepper.</summary>
			ClockwiseCounterClockwise = 1,
			/// <summary>Bipolar PWM servo.</summary>
			BipolarPWM = 2,
			/// <summary>Unipolar PWM servo.</summary>
			UnipolarPWM = 3,
            /// <summary>Piezo.</summary>
            Piezo = 4,
            /// <summary>Sine commutation servo.</summary>
			SineCom = 0x8000
		}

		/// <summary>Flags for Get/SetServoOutputPhase() phasing parameter.</summary>
		public enum Phase : short
		{
			/// <summary>Normal phasing.</summary>
			Standard = 0,
			/// <summary>Encoder direction is reversed from motor direction.</summary>
			Reverse = 1
		}

		/// <summary>Flags for Get/SetProfile() type parameter.</summary>
		public enum ProfileType : short
		{
			/// <summary>Can't determine current setting.</summary>
			Unknown = 0,
			/// <summary>Trapezoidal profile (default).</summary>
			Trapezoid = 1,
			/// <summary>S-Curve profile.</summary>
			SCurve = 2,
			/// <summary>Parabolic profile.</summary>
			Parabolic = 4
		}

		/// <summary>Flags for ArcCenter(), ArcEndAngle(), and InterruptOnPosition().</summary>
		public enum PositionType : int
		{
			/// <summary>Position is absolute.</summary>
			Absolute = 0,
			/// <summary>Position is relative to current position.</summary>
			Relative = 1
		}

		/// <summary>Flags for UpdateRate member of FilterEx structure.</summary>
		public enum UpdateRate : int
		{
			/// <summary>Can't determine current setting.</summary>
			Unknown = 0,
			/// <summary>Low speed mode.</summary>
			Low = 1,
			/// <summary>Medium speed mode.</summary>
			Medium = 2,
			/// <summary>High speed mode.</summary>
			High = 4
		}

		/// <summary>
		/// Constants for DecodeStatusEx() and DecodeStatus().
		/// </summary>
		[Flags()]
			public enum StatusBit : int
		{
			/// <summary>Controller busy.</summary>
			Busy = 0,
			/// <summary>Axis enabled.</summary>
			Enabled = 1, 
			/// <summary>Axis in velocity mode.</summary>
			VelocityMode = 2, 
			/// <summary>Trajectory complete (move finished).</summary>
			TrajectoryComplete = 3, 
			/// <summary>Moving in negative direction.</summary>
			DirectionNeg = 4, 
			/// <summary>Jog mode enabled.</summary>
			JoggingEnabled = 5, 
			/// <summary>Axis is homed.</summary>
			Homed = 6, 
			/// <summary>Axis has detected an error (limit, following, etc.).</summary>
			Error = 7, 
			/// <summary>Axis is looking for the index pulse.</summary>
			LookingIndex = 8, 
			/// <summary>Axis is looking for the edge pulse.</summary>
			LookingEdge = 9,
			/// <summary>Breakpoint position reached.</summary>
			Breakpoint = 10, 
			/// <summary>Following erro limit has been exceeded.</summary>
			FollowingError = 11, 
			/// <summary>Amplifier fault detection enabled.</summary>
			AmpFaultEnable = 12, 
			/// <summary>Amplifier fault error.</summary>
			AmpFault = 13, 
			/// <summary>Positive hard limit enabled.</summary>
			PlusLimitEnabled = 14, 
			/// <summary>Positive hard limit tripped.</summary>
			PlusLimitTripped = 15, 
			/// <summary>Negative hard limit enabled.</summary>
			MinusLimitEnabled = 16, 
			/// <summary>Negative hard limit tripped.</summary>
			MinusLimitTripped = 17, 
			/// <summary>Positive soft limit enabled.</summary>
			PlusSoftEnabled = 18, 
			/// <summary>Positive soft limit tripped.</summary>
			PlusSoftTripped = 19,
			/// <summary>Negative soft limit enabled.</summary>
			MinusSoftEnabled = 20, 
			/// <summary>Negative soft limit tripped.</summary>
			MinusSoftTripped = 21, 
			/// <summary>Index input bit state.</summary>
			InputIndex = 22, 
			/// <summary>Home input bit state.</summary>
			InputHome = 23, 
			/// <summary>Amplifier fault input bit state.</summary>
			InputAmpFault = 24, 
			/// <summary>Auxliary input bit state.</summary>
			InputAuxiliary = 25, 
			/// <summary>Positive hard limit input bit state.</summary>
			InputPlusLimit = 26,
			/// <summary>Negative hard limit input bit state.</summary>
			InputMinusLimit = 27, 
			/// <summary>User 1 input bit state.</summary>
			InputUser1 = 28, 
			/// <summary>User 2 input bit state.</summary>
			InputUser2 = 29,
			/// <summary>Servo ouput phasing reversed.</summary>
			Phase = 30,
			/// <summary>Stepper motor is in full step mode.</summary>
			FullStep = 31,
			/// <summary>Stepper motor is in half step mode.</summary>
			HalfStep = 32,
			/// <summary>Axis is jogging.</summary>
			Jogging = 33,
			/// <summary>Positive jog button is enabled.</summary>
			PlusJogEnabled = 34,
			/// <summary>Positive jog button is active.</summary>
			PlusJogActive = 35,
			/// <summary>Negative jog button is enabled.</summary>
			MinusJogEnabled = 36,
			/// <summary>Negative jog button is active.</summary>
			MinusJogActive = 37,
			/// <summary>Positive jog input state.</summary>
			InputPlusJog = 38,
			/// <summary>Negative jog input state.</summary>
			InputMinusJog = 39,
			/// <summary>Axis is stopping.</summary>
			Stopping = 40,
			/// <summary>Programmed (desired) axis direction.</summary>
			ProgrammedDirection = 41,
			/// <summary>Axis is at target.</summary>
			AtTarget = 42,
			/// <summary>Axis is accelerating (DC2 only).</summary>
			Accelerating = 43,
			/// <summary>Axis is in position mode.</summary>
			ModePosition = 44,
			/// <summary>Axis is in torque mode.</summary>
			ModeTorque = 45,
			/// <summary>Axis is in arc mode (DC2 only).</summary>
			ModeArc = 46,
			/// <summary>Axis is in contour mode.</summary>
			ModeContour = 47,
			/// <summary>Axis is in slave mode (DC2 only).</summary>
			ModeSlave = 48,
			/// <summary>Abort motion on limit event.</summary>
			LimitAbort = 49,
			/// <summary>Stop motion on limit event.</summary>
			LimitStop = 50,
			/// <summary>Position captured (DC2 only).</summary>
			Capture = 51,
			/// <summary>Position record enabled (DC2 only).</summary>
			Record = 52,
			/// <summary>Synchronization on (DC2 only).</summary>
			Sync = 53,
			/// <summary>Axis is in linear mode (DC2 only).</summary>
			ModeLinear = 54,
			/// <summary>Index signal found (servo).</summary>
			IndexFound = 55,
			/// <summary>Position capture mode status.</summary>
			PositionCapture = 56,
			/// <summary>NULL.</summary>
			Null = 57,
			/// <summary>Edge signal found (stepper).</summary>
			EdgeFound = 58,
			/// <summary>Primary encoder fault detected.</summary>
			PrimaryEncoderFault = 59,
			/// <summary>Auxiliary encoder fault detected.</summary>
			AuxiliaryEncoderFault = 60,
			/// <summary>Looking for the auxiliary encoder index pulse.</summary>
			LookingForAuxIndex = 61,
			/// <summary>Found the auxiliary encoder index pulse.</summary>
			AuxIndexFound = 62
		}

		/// <summary>Flags for Current member of MotionEx structure().</summary>
		public enum StepperCurrent : int
		{
			/// <summary>Full current.</summary>
			Full = 1,
			/// <summary>Half current.</summary>
			Half = 2
		}

		/// <summary>Flags for StepSize member of MotionEx structure().</summary>
		public enum StepSize : int
		{
			/// <summary>Full step mode.</summary>
			Full = 1,
			/// <summary>Half step (micro-step) mode.</summary>
			Half = 2
		}

		//
		// -- Structure declarations -------------------------------------------
		//

		/// <summary>
		/// GetAxisConfiguration() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct AxisConfig 
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)]   public Int32  cbSize;
			/// <summary>Module/Axis type identifier. One of the ModuleType enum values.</summary>
			[FieldOffset(4)] public ModuleType ModuleType;
			/// <summary>Module location.</summary>
			[FieldOffset(8)] public Int32 ModuleLocation;
			/// <summary>Generic motor type (servo or stepper).</summary>
			[FieldOffset(12)] public MotorType MotorType;
			/// <summary>
			/// Capture modes supported, combination of CaptureType.Actual, CaptureType.Error, 
			/// CaptureType.Optimal, CaptureType.Torque, CaptureType.Advanced, CaptureType.Auxiliary,
			/// or CaptureType.Status, OR'ed together.
			/// </summary>
			[FieldOffset(16)] public CaptureType CaptureModes;
			/// <summary>Capture data buffer size.</summary>
			[FieldOffset(20)] public Int32 CapturePoints;
			/// <summary>Supports capture and compare (TRUE/FALSE).</summary>
			[FieldOffset(24)] public Int32 CaptureAndCompare;
			/// <summary>Servo update rate in HIGH speed mode.</summary>
			[FieldOffset(28)] public Double HighRate;
			/// <summary>Servo update rate in MEDIUM speed mode.</summary>
			[FieldOffset(36)] public Double MediumRate;
			/// <summary>Servo update rate in LOW speed mode.</summary>
			[FieldOffset(44)] public Double LowRate;
			/// <summary>Minimum stepper motor step rate in HIGH speed mode.</summary>
			[FieldOffset(52)] public Double HighStepMin;
			/// <summary>Maximum stepper motor step rate in HIGH speed mode.</summary>
			[FieldOffset(60)] public Double HighStepMax;
			/// <summary>Minimum stepper motor step rate in MEDIUM speed mode.</summary>
			[FieldOffset(68)] public Double MediumStepMin;
			/// <summary>Maximum stepper motor step rate in MEDIUM speed mode.</summary>
			[FieldOffset(76)] public Double MediumStepMax;
			/// <summary>Minimum stepper motor step rate in LOW speed mode.</summary>
			[FieldOffset(84)] public Double LowStepMin;
			/// <summary>Maximum stepper motor step rate in LOW speed mode.</summary>
			[FieldOffset(92)] public Double LowStepMax;
			/// <summary>Axis supports auxiliary encoder (TRUE/FALSE).</summary>
			[FieldOffset(104)] public Int32 AuxEncoder;
		}

		/// <summary>
		/// SetCommutation() data structure.
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct Commutation 
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)] public Int32 cbSize;
			/// <summary>Phase A setting, in degrees.</summary>
			[FieldOffset(4)] public Double PhaseA;
			/// <summary>Phase B setting, in degrees.</summary>
			[FieldOffset(12)] public Double PhaseB;
			/// <summary>Commutation divisor.</summary>
			[FieldOffset(20)] public Int32 Divisor;
			/// <summary>Commutation prescale factor.</summary>
			[FieldOffset(24)] public Int32 PreScale;
			/// <summary>Commutation repeat count.</summary>
			[FieldOffset(28)] public Int32 Repeat;
		}

		/// <summary>
		/// GetContourConfig() / SetContourConfig() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct Contour 
		{
			/// <summary>Contour path vector acceleration value.</summary>
			[FieldOffset(0)] public Double VectorAccel;
			/// <summary>Contour path vector deceleration value.</summary>
			[FieldOffset(8)] public Double VectorDecel;
			/// <summary>Contour path vector velocity.</summary>
			[FieldOffset(16)] public Double VectorVelocity;
			/// <summary>Velocity override for vector velocity.</summary>
			[FieldOffset(24)] public Double VelocityOverride;
		}

		/// <summary>
		/// GetFilterConfigEx() / SetFilterConfigEx() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct FilterEx 
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)] public Int32 cbSize;
            /// <summary>Proportional gain PID term (Kp).</summary>
            [FieldOffset(4)] public Double Gain;
            /// <summary>Integral gain PID term (Ki).</summary>
			[FieldOffset(12)] public Double IntegralGain;
            /// <summary>Integration time limit value.</summary>
			[FieldOffset(20)] public Double IntegrationLimit;
            /// <summary>Integral term mode, one of IntegralGainMode.Normal, IntegralGainMode.Freeze, or IntegralGainMode.Zero.</summary>
			[FieldOffset(28)] public IntegralGainMode IntegralOption;
			/// <summary>Derivative gain PID term (Kd).</summary>
			[FieldOffset(32)] public Double DerivativeGain;
            /// <summary>Derivative term sample period.</summary>
			[FieldOffset(40)] public Double DerSamplePeriod;
			/// <summary>Following error limit.</summary>
			[FieldOffset(48)] public Double FollowingError;
			/// <summary>Feed-forward velocity gain.</summary>
			[FieldOffset(56)] public Double VelocityGain;
			/// <summary>Feed-forward acceleration gain.</summary>
			[FieldOffset(64)] public Double AccelGain;
			/// <summary>Feed forward deceleration gain.</summary>
			[FieldOffset(72)] public Double DecelGain;
			/// <summary>Encoder scale factor for closed-loop steppers.</summary>
			[FieldOffset(80)] public Double EncoderScaling;
			/// <summary>Servo feedback loop rate or maximum stepper step rate, one of Rate.Low, Rate.Medium, or Rate.High.</summary>
			[FieldOffset(88)] public UpdateRate UpdateRate;
			/// <summary>Position deadband used to determine when an axis is "At Target".</summary>
			[FieldOffset(92)] public Double PositionDeadband;
			/// <summary>Time period axis must remain within the position deadband to be "At Target".</summary>
			[FieldOffset(100)] public Double DelayAtTarget;
			/// <summary>Programmable offset to the servo output signal zero.</summary>
			[FieldOffset(108)] public Double OutputOffset;
			/// <summary>Deadband about zero point of the servo output signal.</summary>
			[FieldOffset(116)] public Double OutputDeadband;
		}

        /// <summary>
        /// GetJogConfigEx() / SetJogConfigEx() data structure
        /// </summary>
        [StructLayout(LayoutKind.Explicit)]
        public struct JogEx
        {
            /// <summary>Size of structure, in bytes.</summary>
            [FieldOffset(0)]
            public Int32 cbSize;
            /// <summary>Jog mode acceleration.</summary>
            [FieldOffset(4)]
            public Double Acceleration;
            /// <summary>Jog mode minimum velocity.</summary>
            [FieldOffset(12)]
            public Double MinVelocity;
            /// <summary>Jog mode deadband.</summary>
            [FieldOffset(20)]
            public Double Deadband;
            /// <summary>Jog mode gain.</summary>
            [FieldOffset(28)]
            public Double Gain;
            /// <summary>Jog mode offset.</summary>
            [FieldOffset(36)]
            public Double Offset;
            /// <summary>Jog mode channel.</summary>
            [FieldOffset(44)]
            public Int32 Channel;
        }

		/// <summary>
		/// GetJogConfig() / SetJogConfig() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct Jog 
		{
			/// <summary>Jog mode acceleration.</summary>
			[FieldOffset(0)] public Double Acceleration;
			/// <summary>Jog mode minimum velocity.</summary>
			[FieldOffset(8)] public Double MinVelocity;
			/// <summary>Jog mode deadband.</summary>
			[FieldOffset(16)] public Double Deadband;
			/// <summary>Jog mode gain.</summary>
			[FieldOffset(24)] public Double Gain;
			/// <summary>Jog mode offset.</summary>
			[FieldOffset(32)] public Double Offset;
		}

		/// <summary>
		/// GetMotionConfigEx() / SetMotionConfigEx() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct MotionEx 
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)] public Int32 cbSize;
			/// <summary>Acceleration value.</summary>
			[FieldOffset(4)] public Double Acceleration;
			/// <summary>Deceleration value.</summary>
			[FieldOffset(12)] public Double Deceleration;
			/// <summary>Programmed velocity value.</summary>
			[FieldOffset(20)] public Double Velocity;
			/// <summary>Minimum velocity for stepper axes.</summary>
			[FieldOffset(28)] public Double MinVelocity;
			/// <summary>Programmed direction for velocity mode operation, one of Dircection.Positive, Direction.Negative.</summary>
			[FieldOffset(36)] public MoveDirection Direction;
			/// <summary>Torque value.</summary>
			[FieldOffset(40)] public Double Torque;
			/// <summary>Position deadband used to determine when an axis is "At Target".</summary>
			[FieldOffset(48)] public Double Deadband;
			/// <summary>Time period axis must remain within the position deadband to be "At Target".</summary>
			[FieldOffset(56)] public Double DeadbandDelay;
			/// <summary>Stepper motor step size, one of StepperStepSize.Full or StepperStepSize.Half.</summary>
			[FieldOffset(64)] public StepSize StepSize;
			/// <summary>Stepper motor current, one of StepperCurrent.Full or StepperCurrent.Half.</summary>
			[FieldOffset(68)] public StepperCurrent Current;
			/// <summary>Hard (physical) limit switch configuration.</summary>
			[FieldOffset(72)] public LimitMode HardLimitMode;
			/// <summary>Hard (software) limit switch configuration.</summary>
			[FieldOffset(76)] public LimitMode SoftLimitMode;
			/// <summary>Soft limit low position value.</summary>
			[FieldOffset(80)] public Double SoftLimitLow;
			/// <summary>Soft limit high position value.</summary>
			[FieldOffset(88)] public Double SoftLimitHigh;
			/// <summary>Enable/disable amp fault input.</summary>
			[FieldOffset(96)] public Int32 EnableAmpFault;
		}

		/// <summary>
		/// GetConfiguration() data structure
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct ParamEx 
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)] public Int32 cbSize;
			/// <summary>Controller ID.</summary>
			[FieldOffset(4)] public Int32 ID;
			/// <summary>Controller type, one of the ControllerType enum values.</summary>
			[FieldOffset(8)] public ControllerType ControllerType;
			/// <summary>Number of installed axes.</summary>
			[FieldOffset(12)] public Int32 NumberAxes;
			/// <summary>Maximum number of axes supported by this controller.</summary>
			[FieldOffset(16)] public Int32 MaximumAxes;
			/// <summary>Maximum number of installable modules supported by this controller.</summary>
			[FieldOffset(20)] public Int32 MaximumModules;
			/// <summary>Best numerical precision of controller, one of _TYPE_LONG or _TYPE_DOUBLE.</summary>
			[FieldOffset(24)] public Int32 Precision;
			/// <summary>Number of digital I/O channels.</summary>
			[FieldOffset(28)] public Int32 DigitalIO;
			/// <summary>Numer of analog inputs.</summary>
			[FieldOffset(32)] public Int32 AnalogInput;
			/// <summary>Number of analog outputs.</summary>
			[FieldOffset(36)] public Int32 AnalogOutput;
			/// <summary>Size of point storage available to LearnPoint() (number of points).</summary>
			[FieldOffset(40)] public Int32 PointStorage;
			/// <summary>Controller supports scaling.</summary>
			[FieldOffset(44)] public Int32 CanDoScaling;
			/// <summary>Controller supports contouring.</summary>
			[FieldOffset(48)] public Int32 CanDoContouring;
			/// <summary>Controller supports multiple accel/decel profiles.</summary>
			[FieldOffset(52)] public Int32 CanChangeProfile;
			/// <summary>Controller supports adjustable servo loop rates.</summary>
			[FieldOffset(56)] public Int32 CanChangeRates;
			/// <summary>Controller supports soft limits.</summary>
			[FieldOffset(60)] public Int32 SoftLimits;
			/// <summary>Controller supports Multi-tasking.</summary>
			[FieldOffset(64)] public Int32 MultiTasking;
			/// <summary>Controller supports amp fault detection.</summary>
			[FieldOffset(68)] public Int32 AmpFault;
			/// <summary>Analog input minimum voltage.</summary>
			[FieldOffset(72)] public Double AnalogInpMin;
			/// <summary>Analog input maximum voltage.</summary>
			[FieldOffset(80)] public Double AnalogInpMax;
			/// <summary>Analog input resolutiuon (bits).</summary>
			[FieldOffset(88)] public Int32  AnalogInpRes;
			/// <summary>Analog output minimum voltage.</summary>
			[FieldOffset(92)] public Double AnalogOutMin;
			/// <summary>Analog output maximum voltage.</summary>
			[FieldOffset(100)] public Double AnalogOutMax;
			/// <summary>Analog output resolution (bits).</summary>
			[FieldOffset(108)] public Int32 AnalogOutRes;
            /// <summary>Added v4.1.3, can accept line oriented ASCII (e.g. TCP/IP controllers)</summary>
            [FieldOffset(112)] public Int32 LineModeAscii;
		}

		/// <summary>
		/// GetStatusEx data structure.
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct Scaling 
		{
			/// <summary>
			/// This factor acts as a scale factor for servo analog outputs. By calibrating your 
			/// motor/amplifier combination, it is possible to scale the output with Constant so 
			/// that torque settings may be specified directly in ft-lbs.
			/// </summary>
			[FieldOffset(0)] public Double Constant;
			/// <summary>
			/// This offset represents an offset from a servo encoders index pulse to
			/// a zero position.
			/// </summary>
			[FieldOffset(8)] public Double Offset;
			/// <summary>
			/// This factor acts as a multiplier for motion commands time values. The base 
			/// controller time unit is the second, to convert this to minutes set the rate 
			/// to 60.0, to convert to milliseconds rate should be set to 0.001.
			/// </summary>
			[FieldOffset(16)] public Double Rate;
			/// <summary>
			/// This scaling factor is applied to motion parameters to convert from encoder 
			/// counts to real world units.
			/// </summary>
			[FieldOffset(24)] public Double Scale;
			/// <summary>
			/// Specifies that a soft zero should be located this distance from actual zero. 
			/// By moving the soft zero around it is possible to have a series of position 
			/// commands repeated at various spots in the range of travel without modifying 
			/// the position commands. The actual zero position is not changed by this command.
			/// </summary>
			[FieldOffset(32)] public Double Zero;
			/// <summary>
			/// This is the time factor for controller level wait commands. See the discussion of 
			/// the Rate parameter above for more information on setting this value. Note that a 
			/// single Time value is maintained per controller (ie Time is axis independent).
			/// </summary>
			[FieldOffset(40)] public Double Time;
		}

		/// <summary>
		/// DecodeStatusEx() data structure.
		/// </summary>
		[StructLayout(LayoutKind.Explicit)]
			public struct StatusEx
		{
			/// <summary>Size of structure, in bytes.</summary>
			[FieldOffset(0)] public Int32 cbSize;
			/// <summary>Controller primary status word.</summary>
			[FieldOffset(4)] public UInt32 Status;
			/// <summary>Controller primary auxiliary status word. Additional, less frequently used status bits.</summary>
			[FieldOffset(8)] public UInt32 AuxStatus;
			/// <summary>Controller profile generator status word. Used to view current Accel/Decel profiles.</summary>
			[FieldOffset(12)] public UInt32 ProfileStatus;
			/// <summary>Controller mode status word. Current controller operating mode status.</summary>
			[FieldOffset(16)] public UInt32 ModeStatus;
		}

		//
		// -- Function declarations --------------------------------------------
		//

		/// <summary>MCAPI handle accessor.</summary>
		public Int16 Handle
		{
			get { return m_hCtlr; }
            set { m_hCtlr = value; }
        }

		/// <summary>Should Mcapi throw exceptions.</summary>
		public Boolean ThrowExceptions
		{
			get { return m_throw_exceptions; }
			set { m_throw_exceptions = value; }
		}

		[DllImport(mcapidll, EntryPoint = "MCAbort", CharSet = CharSet.Ansi)]
		private static extern void MCAbort(Int16 hCtlr, Int16 axis);
		/// <summary>
		/// Abort the current motion.
		/// </summary>
		/// <param name="axis">Axis number to abort motion for, specify 0 to abort motion for all axes.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Abort(Int16 axis)
		{
			MCAbort(m_hCtlr, axis);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCArcCenter", CharSet = CharSet.Ansi)]
		private static extern Int32 MCArcCenter(Int16 hCtlr, Int16 axis, Int16 type, Double position);
		/// <summary>Specify the center of an arc for contour path motion.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">Position mode, one of PositionType.Absolute or PositionType.Relative.</param>
		/// <param name="position">Center position for this axis.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ArcCenter(Int16 axis, PositionType type, Double position)
		{
			return ProcessException(MCArcCenter(m_hCtlr, axis, Convert.ToInt16(type), position));
		}

		[DllImport(mcapidll, EntryPoint = "MCArcEndAngle", CharSet = CharSet.Ansi)]
		private static extern Int32 MCArcEndAngle(Int16 hCtlr, Int16 axis, Int16 type, Double angle);
		/// <summary>Specify the ending angle of an arc for contour path motion.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">Position mode, one ofPositionType.Absolute or PositionType.Relative.</param>
		/// <param name="angle">Absolute or relative arc ending angle for axis.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ArcEndAngle(Int16 axis, PositionType type, Double angle)
		{
			return ProcessException(MCArcEndAngle(m_hCtlr, axis, Convert.ToInt16(type), angle));
		}

		[DllImport(mcapidll, EntryPoint = "MCArcRadius", CharSet = CharSet.Ansi)]
		private static extern Int32 MCArcRadius(Int16 hCtlr, Int16 axis, Double radius);
		/// <summary>Specify the radius of an arc for contour path motion.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="radius">Arc radius for axis.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ArcRadius(Int16 axis, Double radius)
		{
			return ProcessException(MCArcRadius(m_hCtlr, axis, radius));
		}

		[DllImport(mcapidll, EntryPoint = "MCBlockBegin", CharSet = CharSet.Ansi)]
		private static extern Int32 MCBlockBegin(Int16 hCtlr, BlockType mode, Int32 num);
		/// <summary>Begins a block command sequence. All commands up to the subsequent BlockEnd()
		/// will be included in the block. Block commands include compound commands, macro 
		/// definition commands, contour path motions, and tasks on multitasking controllers.</summary>
		/// <param name="mode">One of BlockModes.Compound, BlockModes.Task, BlockModes.Macro, BlockModes.ResetMacros, BlockModes.Cancel,
		/// BlockModes.User, BlockModes.Linear, BlockModes.Clockwise, or BlockModes.CounterClockwise.</param>
		/// <param name="num">Specifies the macro number for macro blocks, the task number for task blocks, the controlling axis for contour blocks, or the macro types for macro reset.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error BlockBegin(BlockType mode, Int32 num)
		{
			return ProcessException(MCBlockBegin(m_hCtlr, mode, num));
		}

		[DllImport(mcapidll, EntryPoint = "MCBlockEnd", CharSet = CharSet.Ansi)]
		private static extern Int32 MCBlockEnd(Int16 hCtlr, ref UInt32 taskID);
		/// <summary>Ends a block command and transmits the compound command, task, macro, or 
		/// contour path to the controller.</summary>
		/// <param name="taskID">Pointer to variable to hold the Task ID value for BlockType.Task blocks.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error BlockEnd(ref UInt32 taskID)
		{
			return ProcessException(MCBlockEnd(m_hCtlr, ref taskID));
		}

		[DllImport(mcapidll, EntryPoint = "MCCancelTask", CharSet = CharSet.Ansi)]
		private static extern Int32 MCCancelTask(Int16 hCtlr, UInt32 taskID);
		/// <summary>Cancels an executing task on a multi-tasking controller.</summary>
		/// <param name="taskID">Task ID value for the task to be stopped.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error CancelTask(UInt32 taskID)
		{
			return ProcessException(MCCancelTask(m_hCtlr, taskID));
		}

		[DllImport(mcapidll, EntryPoint = "MCCaptureData", CharSet = CharSet.Ansi)]
		private static extern Int32 MCCaptureData(Int16 hCtlr, Int16 axis, Int32 lPoints, Double Period, Double Delay);
		/// <summary>Configures a controller to perform data capture for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="Points">Number of data points to collect.</param>
		/// <param name="Period">Time period between subsequent data point captures in seconds.</param>
		/// <param name="Delay">Delay (dwell) before initial data collection in seconds.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error CaptureData(Int16 axis, Int32 Points, Double Period, Double Delay)
		{
			return ProcessException(MCCaptureData(m_hCtlr, axis, Points, Period, Delay));
		}

		[DllImport(mcapidll, EntryPoint = "MCClose", CharSet = CharSet.Ansi)]
		private static extern Int16 MCClose(Int16 hCtlr);
		/// <summary>Closes the specified motion controller handle.</summary>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Close()
		{
			Int16 ret = 0;
			if (m_hCtlr > 0)
				ret = MCClose(m_hCtlr);
			m_hCtlr = 0;
			return ProcessException((Error)ret);
		}

		[DllImport(mcapidll, EntryPoint = "MCConfigureCompare", CharSet = CharSet.Ansi)]
		private static extern Int32 MCConfigureCompare(Int16 hCtlr, Int16 axis, ref Double values, Int32 num, Double inc, CompareType type, Double period);
		/// <summary>Configures an axis for high-speed position compare mode operation.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="values">Array of compare position values.</param>
		/// <param name="num">Number of compare values.</param>
		/// <param name="inc">Increment between successive compare positions when in evenly-spaced mode.</param>
		/// <param name="type">One of CompareTypes.Disable, CompareTypes.Enable, CompareTypes.Static, 
		/// CompareTypes.Toggle, or CompareTypes.OneShot, optionally OR'ed with CompareTypes.Invert</param>
		/// <param name="period">Period, in seconds, of the one-shot output when one-shot output mode has been selected.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ConfigureCompare(Int16 axis, ref Double values, Int32 num, Double inc, CompareType type, Double period)
		{
			return ProcessException(MCConfigureCompare(m_hCtlr, axis, ref values, num, inc, type, period));
		}

		[DllImport(mcapidll, EntryPoint = "MCConfigureDigitalIO", CharSet = CharSet.Ansi)]
		private static extern Int16 MCConfigureDigitalIO(Int16 hCtlr, Int16 channel, DigitalIOType type);
		/// <summary>Configure a specific digital I/O channel for input or output and for high 
		/// or low true logic.</summary>
		/// <param name="channel">Digital channel number to configure.</param>
		/// <param name="type">Combination of one or more of DigitalIOType.Input, DigitalIOType.Output, DigitalIOType.High, DigitalIOType.Low, 
		/// or DigitalIOType.Latch, OR'ed together.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ConfigureDigitalIO(Int16 channel, DigitalIOType type)
		{
			MCConfigureDigitalIO(m_hCtlr, channel, type);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCContourDistance", CharSet = CharSet.Ansi)]
		private static extern Int32 MCContourDistance(Int16 hCtlr, Int16 axis, Double distance);
		/// <summary>Sets the distance for user defined contour path motions.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="distance">Path distance for user path.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error ContourDistance(Int16 axis, Double distance)
		{
			return ProcessException(MCContourDistance(m_hCtlr, axis, distance));
		}

		[DllImport(mcapidll, EntryPoint = "MCDecodeStatus", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDecodeStatus(Int16 hCtlr, UInt32 status, StatusBit bit);
		/// <summary>Permits you to test flags in the controller status word in a way that is 
		/// independent of the type of controller being inspected.</summary>
		/// <param name="status">Status value returned from a previous call to GetStatus().</param>
		/// <param name="bit">Status bit to decode. Over fifty different status bit flags (not all 
		/// flags are supported by all controllers). Valid Bit constants are declared in Mcapi.StatusBit.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Boolean DecodeStatus(UInt32 status, StatusBit bit)
		{
			return Convert.ToBoolean(MCDecodeStatus(m_hCtlr, status, bit));
		}

		[DllImport(mcapidll, EntryPoint = "MCDecodeStatusEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDecodeStatusEx(Int16 hCtlr, ref StatusEx status, StatusBit bit);
		/// <summary>Test flags in the controller status words in a way that is independent
		/// of the type of controller being inspected.</summary>
		/// <param name="status">Mcapi.StatusEx structure returned from a previous 
		/// call to GetStatusEx().</param>
		/// <param name="bit">Status bit to decode. Over sixty different status bit flags 
		/// (not all flags are supported by all controllers) are defined. Valid bit 
		/// constants are declared in Mcapi.StatusBit.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Boolean DecodeStatusEx(ref StatusEx status, StatusBit bit)
		{
			return Convert.ToBoolean(MCDecodeStatusEx(m_hCtlr, ref status, bit));
		}

		[DllImport(mcapidll, EntryPoint = "MCDirection", CharSet = CharSet.Ansi)]
		private static extern void MCDirection(Int16 hCtlr, Int16 axis, Int16 dir);
		/// <summary>Set the direction of motion when operating in velocity mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="dir">One of MoveDirection.Positive or MoveDirection.Negative.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Direction(Int16 axis, MoveDirection dir)
		{
			MCDirection(m_hCtlr, axis, Convert.ToInt16(dir));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCEdgeArm", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEdgeArm(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Arm the edge capture function for the home input of an open-loop stepper axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">The position the axis will be set to when the edge
		///  signal is detected.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EdgeArm(Int16 axis, Double position)
		{
			return ProcessException(MCEdgeArm(m_hCtlr, axis, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableAxis", CharSet = CharSet.Ansi)]
		private static extern void MCEnableAxis(Int16 hCtlr, Int16 axis, Int16 state);
		/// <summary>Turn the specified axis on or off.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableAxis(Int16 axis, Boolean state)
		{
			MCEnableAxis(m_hCtlr, axis, Convert.ToInt16(state));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableBacklash", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEnableBacklash(Int16 hCtlr, Int16 axis, Double backlash, Int16 state);
		/// <summary>Enable or disable backlash compensation.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="backlash">Amount of backlash compensation to apply. This parameter 
		/// is ignored if state is FALSE.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableBacklash(Int16 axis, Double backlash, Boolean state)
		{
			return ProcessException(MCEnableBacklash(m_hCtlr, axis, backlash, Convert.ToInt16(state)));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableCapture", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEnableCapture(Int16 hCtlr, Int16 axis, Int32 count);
		/// <summary>Begins position capture for the specified axis if count is greater than 
		/// zero, or stops position capture if count is -1.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="count">Set to -1 to disable capture mode, or to a number greater 
		/// than zero to capture that many positions.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableCapture(Int16 axis, Int32 count)
		{
			return ProcessException(MCEnableCapture(m_hCtlr, axis, count));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableCompare", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEnableCompare(Int16 hCtlr, Int16 axis, Int32 count);
		/// <summary>Enables or disables high-speed compare mode for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="count">If the compare operation was configured as one or more 
		/// discrete points then the "count" argument of should be set equal to the 
		/// number of points. If instead the compare operation was configured for fixed 
		/// increment mode set count to 0. To disable compare set count to 1.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableCompare(Int16 axis, Int32 count)
		{
			return ProcessException(MCEnableCompare(m_hCtlr, axis, count));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableDigitalFilter", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEnableDigitalFilter(Int16 hCtlr, Int16 axis, Int32 state);
		/// <summary>Enables or disables the  digital filter capability of advanced motor 
		/// modules, such as the MC300.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableDigitalFilter(Int16 axis, Boolean state)
		{
			return ProcessException(MCEnableDigitalFilter(m_hCtlr, axis, Convert.ToInt32(state)));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableDigitalIO", CharSet = CharSet.Ansi)]
		private static extern void MCEnableDigitalIO(Int16 hCtlr, Int16 channel, Int16 state);
		/// <summary>Turn the specified digital I/O channel on or off.</summary>
		/// <param name="channel">Digital channel number</param>
		/// <param name="state">TRUE to enable channel, FALSE to disable</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableDigitalIO(Int16 channel, Boolean state)
		{
			MCEnableDigitalIO(m_hCtlr, channel, Convert.ToInt16(state));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableEncoderFault", CharSet = CharSet.Ansi)]
		private static extern Int32 MCEnableEncoderFault(Int16 hCtlr, Int16 axis, Encoder type);
		/// <summary>Enables or disables encoder fault detection.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">Set to one of Encoder.Primary, Encoder.Auxiliary, or Encoder.Both.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableEncoderFault(Int16 axis, Encoder type)
		{
			return ProcessException(MCEnableEncoderFault(m_hCtlr, axis, type));
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableGearing", CharSet = CharSet.Ansi)]
		private static extern void MCEnableGearing(Int16 hCtlr, Int16 axis, Int16 master, Double ratio, Int16 state);
		/// <summary>Enables or disables electronic gearing for the specified Axis/Master pair.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="master">Master axis that Axis is to follow.</param>
		/// <param name="ratio">Ratio at which Axis is to reproduce Masters motions.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableGearing(Int16 axis, Int16 master, Double ratio, Boolean state)
		{
			MCEnableGearing(m_hCtlr, axis, master, ratio, Convert.ToInt16(state));
			return ProcessException();
		}

        /// <summary>Callback function (delegate) for status interrupts.</summary>
        public delegate void InterruptCallback(Int32 hWnd, Int16 hCtlr, Int16 axis, UInt32 status);

        [DllImport(mcapidll, EntryPoint = "MCEnableInterrupt", CharSet = CharSet.Ansi)]
        private static extern Int32 MCEnableInterrupt(Int32 hWnd, Int16 hCtlr, Int16 axis, UInt32 mask, System.IntPtr callback);
        /// <summary>Enables or disables status interrupts.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="mask">Mask of status word bits that can cause a status interrupt.</param>
        /// <param name="callback">Callback function (delegate) that will process the interrupt.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error EnableInterrupt(Int16 axis, UInt32 mask, InterruptCallback callback)
        {
            return ProcessException(MCEnableInterrupt(0, m_hCtlr, axis, mask, Marshal.GetFunctionPointerForDelegate(callback)));
        }

		[DllImport(mcapidll, EntryPoint = "MCEnableJog", CharSet = CharSet.Ansi)]
		private static extern void MCEnableJog(Int16 hCtlr, Int16 axis, Int16 state);
		/// <summary>Enables or disables jogging.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableJog(Int16 axis, Boolean state)
		{
			MCEnableJog(m_hCtlr, axis, Convert.ToInt16(state));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCEnableSync", CharSet = CharSet.Ansi)]
		private static extern void MCEnableSync(Int16 hCtlr, Int16 axis, Int16 state);
		/// <summary>Enables or disables synchronized motion for contour path motion.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="state">TRUE to enable, FALSE to disable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error EnableSync(Int16 axis, Boolean state)
		{
			MCEnableSync(m_hCtlr, axis, Convert.ToInt16(state));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCFindAuxEncIdx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCFindAuxEncIdx(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Initialize a motor's auxiliary encoder at a given position, relative to 
		/// the auxiliary encoder index input.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">The current position of the auxiliary encoder will be 
		/// set to this position when the index signal is sensed.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error FindAuxEncIdx(Int16 axis, Double position)
		{
			return ProcessException(MCFindAuxEncIdx(m_hCtlr, axis, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCFindEdge", CharSet = CharSet.Ansi)]
		private static extern Int32 MCFindEdge(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Initialize a stepper motor at a given position, relative 
		/// to the coarse home input.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">The current position of the axis will be set to this
		///  position when the edge signal is sensed.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error FindEdge(Int16 axis, Double position)
		{
			return ProcessException(MCFindEdge(m_hCtlr, axis, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCFindIndex", CharSet = CharSet.Ansi)]
		private static extern Int32 MCFindIndex(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Initialize a servo motor at a given position, relative to the index input.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">The position where the encoder index pulse occurred 
		/// for the axis will be set to Position after a call to EnableAxis().</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error FindIndex(Int16 axis, Double position)
		{
			return ProcessException(MCFindIndex(m_hCtlr, axis, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetAccelerationEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetAccelerationEx(Int16 hCtlr, Int16 axis, ref Double acceleration);
		/// <summary>Reads the current programmed acceleration value, in whatever units the 
		/// axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="acceleration">Current acceleration upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetAccelerationEx(Int16 axis, ref Double acceleration)
		{
			return ProcessException(MCGetAccelerationEx(m_hCtlr, axis, ref acceleration));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetAnalogEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetAnalogEx(Int16 hCtlr, Int32 channel, ref UInt32 analog_val);
		/// <summary>Reads the current input state of the specified analog input channel.</summary>
		/// <param name="channel">Channel number.</param>
		/// <param name="analog_val">Analog reading upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetAnalogEx(Int32 channel, ref UInt32 analog_val)
		{
			return ProcessException(MCGetAnalogEx(m_hCtlr, channel, ref analog_val));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetAuxEncIdxEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetAuxEncIdxEx(Int16 hCtlr, Int16 axis, ref Double index);
		/// <summary>Reads the position where the auxiliary encoder's index pulse was observed.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="index">Auxiliary encoder index position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetAuxEncIdxEx(Int16 axis, ref Double index)
		{
			return ProcessException(MCGetAuxEncIdxEx(m_hCtlr, axis, ref index));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetAuxEncPosEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetAuxEncPosEx(Int16 hCtlr, Int16 axis, ref Double position);
		/// <summary>Reads the current position of the auxiliary encoder.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">Auxiliary encoder position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetAuxEncPosEx(Int16 axis, ref Double position)
		{
			return ProcessException(MCGetAuxEncPosEx(m_hCtlr, axis, ref position));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetAxisConfiguration", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetAxisConfiguration(Int16 hCtlr, Int16 axis, ref AxisConfig axiscfg);
		/// <summary>Reads the configuration for the specified axis. Configuration information 
		/// includes the axis type, servo motor update rates, stepper motor step rates, etc.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="axiscfg">An AXISCONFIG structure that will contain the configuration 
		/// information upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetAxisConfiguration(Int16 axis, ref AxisConfig axiscfg)
		{
			return ProcessException(MCGetAxisConfiguration(m_hCtlr, axis, ref axiscfg));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetBreakpointEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetBreakpointEx(Int16 hCtlr, Int16 axis, ref Double breakpoint);
		/// <summary>Reads the current breakpoint position as set by the WaitForPosition() 
		/// or WaitForRelative() command.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="breakpoint">Breakpoint position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetBreakpointEx(Int16 axis, ref Double breakpoint)
		{
			return ProcessException(MCGetBreakpointEx(m_hCtlr, axis, ref breakpoint));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetCaptureData", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetCaptureData(Int16 hCtlr, Int16 axis, CaptureType type, Int32 start, Int32 points, [In, Out] Double[] data);
		/// <summary>Retrieves data collected following the most recent CaptureData() call.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">Specifies data to retrieve, one of CaptureTypes.Actual, CaptureTypes.Error,
		/// CaptureTypes.Optimal, CaptureTypes.Torque, CaptureTypes.Auxiliary, or CaptureTypes.Status.</param>
		/// <param name="start">Index of the first data point to retrieve. The index is zero based.</param>
		/// <param name="points">Total number of data points to retrieve.</param>
		/// <param name="data">Array of Doubles large enough to hold requested capture data.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetCaptureData(Int16 axis, CaptureType type, Int32 start, Int32 points, [In, Out] Double[] data)
		{
			return ProcessException(MCGetCaptureData(m_hCtlr, axis, type, start, points, data));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetCaptureSettings", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetCaptureSettings(Int16 hCtlr, Int16 axis, ref Int32 points, ref Double period, ref Double delay, ref Int32 index);
		/// <summary>Get the current data recording settings for the specified axis..</summary>
		/// <param name="axis">Axis number.</param>
        /// <param name="points">Number of data points commanded to collect in last capture.</param>
        /// <param name="period">Time period between subsequent data point captures commanded in last capture.</param>
        /// <param name="delay">Delay (dwell) before initial data collection in last capture.</param>
        /// <param name="index">While capturing data the index will reflect the currently acquiring data point (MCGetCount() 
        /// may also be used to retrieve the index.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error MCGetCaptureSettings(Int16 axis, ref Int32 points, ref Double period, ref Double delay, ref Int32 index)
		{
			return ProcessException(MCGetCaptureSettings(m_hCtlr, axis, ref points, ref period, ref delay, ref index));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetConfigurationEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetConfigurationEx(Int16 hCtlr, ref ParamEx param);
		/// <summary>Reads the configuration information for the specified controller. Configuration 
		/// information includes the controller type, number and type of installed motor
		/// modules, and if the controller supports scaling, contouring, etc.</summary>
		/// <param name="param">An PARAMEX structure that receives the configuration information.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetConfigurationEx(ref ParamEx param)
		{
			return ProcessException(MCGetConfigurationEx(m_hCtlr, ref param));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetContourConfig", CharSet = CharSet.Ansi)]
		private static extern Int16 MCGetContourConfig(Int16 hCtlr, Int16 axis, ref Contour contour);
		/// <summary>Reads the contouring configuration settings.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="contour">An CONTOUR structure that receives the settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetContourConfig(Int16 axis, ref Contour contour)
		{
			MCGetContourConfig(m_hCtlr, axis, ref contour);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCGetCount", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetCount(Int16 hCtlr, Int16 axis, CountType type, ref Int32 count);
		/// <summary>Reads various count values from the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">One of CountType.Capture, CountType.Compare, CountType.Contour, CountType.Filter, or CountType.CountType.FilterMax.</param>
		/// <param name="count">Count upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetCount(Int16 axis, CountType type, ref Int32 count)
		{
			return ProcessException(MCGetCount(m_hCtlr, axis, type, ref count));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCGetDecelerationEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetDecelerationEx(Int16 hCtlr, Int16 axis, ref Double deceleration);
		/// <summary>Reads the current programmed deceleration value, in whatever units the 
		/// axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="deceleration">Deceleration upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetDecelerationEx(Int16 axis, ref Double deceleration)
		{
			return ProcessException(MCGetDecelerationEx(m_hCtlr, axis, ref deceleration));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetDigitalFilter", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetDigitalFilter(Int16 hCtlr, Int16 axis, [In, Out] Double[] coeff, Int32 num, ref Int32 actual);
		/// <summary>Reads the digital filter coefficients.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="coeff">Array of Doubles large enough to hold retrieved coefficients, must be 
		/// num elements long (or longer). If this pointer is NULL no coefficients are retturned.</param>
		/// <param name="num">Number of coefficients to retrieve, cannot be larger than 
		/// the maximum digital filter size supported by the controller.</param>
		/// <param name="actual">Integer that will be set equal to the number of valid coefficients 
		/// currently loaded for this axis. If this pointer is NULL no value is returned.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetDigitalFilter(Int16 axis, [In, Out] Double[] coeff, Int32 num, ref Int32 actual)
		{
			return ProcessException(MCGetDigitalFilter(m_hCtlr, axis, coeff, num, ref actual));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetDigitalIOEx", CharSet = CharSet.Ansi)]
        private static extern Int32 MCGetDigitalIOEx(Int16 hCtlr, Int16 channel, ref UInt32 state);
		/// <summary>Reads the current state of the specified digital I/O channel or all channels.</summary>
		/// <param name="channel">Channel number.</param>
        /// <param name="state">On return holds state of channel (1=on / 0=off), or 32-bit value
        /// representing all 32 digital I/O channels.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error GetDigitalIOEx(Int16 channel, ref UInt32 state)
		{
            return ProcessException(MCGetDigitalIOEx(m_hCtlr, channel, ref state));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetDigitalIOConfig", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetDigitalIOConfig(Int16 hCtlr, Int16 channel, ref DigitalIOType type);
		/// <summary>Reads the current configuration (inp/out/high/low) of the specified 
		/// digital I/O channel.</summary>
		/// <param name="channel">Channel number.</param>
		/// <param name="type">Combination of one or more of DigitalIOType.Input, DigitalIOType.Output, DigitalIOType.High, DigitalIOType.Low, 
		/// DigitalIOType.Latch, DigitalIOType.Fixed, DigitalIOType.Latchable, or DigitalIOType.Stepper, OR'ed together.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetDigitalIOConfig(Int16 channel, ref DigitalIOType type)
		{
			return ProcessException(MCGetDigitalIOConfig(m_hCtlr, channel, ref type));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetError", CharSet = CharSet.Ansi)]
		private static extern Int16 MCGetError(Int16 hCtlr);
		/// <summary>Returns the most recent error code for the controller.</summary>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetError()
		{
			return (Error)MCGetError(m_hCtlr);
		}

		[DllImport(mcapidll, EntryPoint = "MCGetFilterConfigEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetFilterConfigEx(Int16 hCtlr, Int16 axis, ref FilterEx filter);
		/// <summary>Reads the current PID filter configuration for a servo motor or the closed-loop 
		/// configuration for a stepper motor operating in closed-loop mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="filter">An FILTEREX structure that receives the PID filter 
		/// configuration information upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetFilterConfigEx(Int16 axis, ref FilterEx filter)
		{
			return ProcessException(MCGetFilterConfigEx(m_hCtlr, axis, ref filter));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetFollowingError", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetFollowingError(Int16 hCtlr, Int16 axis, ref Double error);
		/// <summary>Reads the current following error (difference between the actual and 
		/// the optimal positions).</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="error">Following error upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetFollowingError(Int16 axis, ref Double error)
		{
			return ProcessException(MCGetFollowingError(m_hCtlr, axis, ref error));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetGain", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetGain(Int16 hCtlr, Int16 axis, ref Double gain);
		/// <summary>Reads the current gain setting.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="gain">Gain upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetGain(Int16 axis, ref Double gain)
		{
			return ProcessException(MCGetGain(m_hCtlr, axis, ref gain));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetIndexEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetIndexEx(Int16 hCtlr, Int16 axis, ref Double index);
		/// <summary>Reads the position where the encoder index pulse was observed, in 
		/// whatever units the axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="index">Index position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetIndexEx(Int16 axis, ref Double index)
		{
			return ProcessException(MCGetIndexEx(m_hCtlr, axis, ref index));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetInstalledModules", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetInstalledModules(Int16 hCtlr, [In, Out] Int32[] modules, Int32 size);
		/// <summary>Enumerates the types of modules (axes) installed on the motion controller.</summary>
		/// <param name="modules">Array of integers, filled with module types on return.</param>
		/// <param name="size">Size of Modules array (number of integers).</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetInstalledModules([In, Out] Int32[] modules, Int32 size)
		{
			return ProcessException(MCGetInstalledModules(m_hCtlr, modules, size));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetJogConfig", CharSet = CharSet.Ansi)]
		private static extern Int16 MCGetJogConfig(Int16 hCtlr, Int16 axis, ref Jog jog);
		/// <summary>Reads the current jogging configuration.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="jog">A JOG structure that will contain jog configuration 
		/// information upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetJogConfig(Int16 axis, ref Jog jog)
		{
			MCGetJogConfig(m_hCtlr, axis, ref jog);
			return ProcessException();
		}

        [DllImport(mcapidll, EntryPoint = "MCGetJogConfigEx", CharSet = CharSet.Ansi)]
        private static extern Int32 MCGetJogConfigEx(Int16 hCtlr, Int16 axis, ref JogEx jog);
        /// <summary>Reads the current jogging configuration.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="jog">A JOGEX structure that will contain jog configuration 
        /// information upon function return.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error GetJogConfigEx(Int16 axis, ref JogEx jog)
        {
            return ProcessException(MCGetJogConfigEx(m_hCtlr, axis, ref jog));
        }

		[DllImport(mcapidll, EntryPoint = "MCGetLimits", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetLimits(Int16 hCtlr, Int16 axis, ref Int16 hardmode, ref Int16 softmode, ref Double softlimitminus, ref Double softlimitplus);
		/// <summary>Reads the current hard and soft limit settings.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="hardmode">One or more of LimitMode.Off, LimitMode.Plus, LimitMode.Minus, 
		/// LimitMode.Both, LimitMode.Abrupt LimitMode.Smooth and LimitMode.Invert, OR'ed together.</param>
		/// <param name="softmode">See options for "hardmode", above.</param>
		/// <param name="softlimitminus">Negative limit value for soft limits upon function return.</param>
		/// <param name="softlimitplus">Positive limit value for soft limits upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetLimits(Int16 axis, ref LimitMode hardmode, ref LimitMode softmode, ref Double softlimitminus, ref Double softlimitplus)
		{
			Int16 hlm = 0, slm = 0;
			Int32 ret = MCGetLimits(m_hCtlr, axis, ref hlm, ref slm, ref softlimitminus, ref softlimitplus);

			hardmode = (LimitMode)hlm;
			softmode = (LimitMode)slm;
			return ProcessException(ret);
		}
		
		[DllImport(mcapidll, EntryPoint = "MCGetModuleInputMode", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetModuleInputMode(Int16 hCtlr, Int16 axis, ref InputMode mode);
		/// <summary>Reads the current axis input mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of the Mcapi.InputMode constants upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetModuleInputMode(Int16 axis, ref InputMode mode)
		{
			return ProcessException(MCGetModuleInputMode(m_hCtlr, axis, ref mode));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetModuleOutputMode", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetModuleOutputMode(Int16 hCtlr, Int16 axis, ref OutputMode mode);
		/// <summary>Reads the current axis output mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of OutputMode.Bipolar, OutputMode.Unipolar, OutputMode.BipolarPWM,
		/// OutputMode.UnipolarPWM, OutputMode.PulseAndDirection, or OutputMode.ClockwiseCounterClockwise.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetModuleOutputMode(Int16 axis, ref OutputMode mode)
		{
			return ProcessException(MCGetModuleOutputMode(m_hCtlr, axis, ref mode));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetMotionConfigEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetMotionConfigEx(Int16 hCtlr, Int16 axis, ref MotionEx motion);
		/// <summary>Reads the current axis motion configuration.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="motion">An MOTIONEX structure that contains motion configuration
		/// information upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetMotionConfigEx(Int16 axis, ref MotionEx motion)
		{
			return ProcessException(MCGetMotionConfigEx(m_hCtlr, axis, ref motion));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCGetOperatingMode", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetOperatingMode(Int16 hCtlr, Int16 axis, ref OperatingMode mode);
		/// <summary>Reads the axis current operating mode (position, velocity, gain, torque, etc.).</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of the Mcapi.OperatingMode constants upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetOperatingMode(Int16 axis, ref OperatingMode mode)
		{
			return ProcessException(MCGetOperatingMode(m_hCtlr, axis, ref mode));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetOptimalEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetOptimalEx(Int16 hCtlr, Int16 axis, ref Double optimal);
		/// <summary>Reads the current optimal position from the trajectory generator, in 
		/// whatever units the axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="optimal">Optimal position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetOptimalEx(Int16 axis, ref Double optimal)
		{
			return ProcessException(MCGetOptimalEx(m_hCtlr, axis, ref optimal));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetPositionEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetPositionEx(Int16 hCtlr, Int16 axis, ref Double position);
		/// <summary>Reads the current position, in whatever units the axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">Current axis position upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetPositionEx(Int16 axis, ref Double position)
		{
			return ProcessException(MCGetPositionEx(m_hCtlr, axis, ref position));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetProfile", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetProfile(Int16 hCtlr, Int16 axis, ref ProfileType profile);
		/// <summary>Reads the axis current acceleration/deceleration profile type.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="profile">One of ProfileType.Unknown, ProfileType.Trapezoidal, ProfileType.SCurve, 
		/// or ProfileType.Parabolic upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetProfile(Int16 axis, ref ProfileType profile)
		{
			return ProcessException(MCGetProfile(m_hCtlr, axis, ref profile));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetRegister", CharSet = CharSet.Ansi)]
		private static unsafe extern Int32 MCGetRegister(Int16 hCtlr, Int32 reg, void* regvalue, DataType type);
		/// <summary>Reads the value of the specified motion controller general purpose register.</summary>
		/// <param name="reg">Controller register number to read from.</param>
		/// <param name="regvalue">Variable that will hold the register contents upon function return.</param>
		/// <param name="type">Data type, on of the DataType enum values. Must match 
		/// the type of the regvalue variable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public unsafe Error GetRegister(Int32 reg, void* regvalue, DataType type)
		{
			return ProcessException(MCGetRegister(m_hCtlr, reg, regvalue, type));
		}

		/// <summary>Reads the value of the specified motion controller general purpose register.</summary>
		/// <param name="reg">Controller register number to read from.</param>
		/// <param name="regvalue">Variable that will hold the register contents upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetRegister(Int32 reg, ref Int32 regvalue)
		{
			unsafe 
			{
				fixed (void* p = &regvalue) return GetRegister(reg, p, DataType.Long);
			}
		}

		/// <summary>Reads the value of the specified motion controller general purpose register.</summary>
		/// <param name="reg">Controller register number to read from.</param>
		/// <param name="regvalue">Variable that will hold the register contents upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetRegister(Int32 reg, ref Single regvalue)
		{
			unsafe 
			{
				fixed (void* p = &regvalue) return GetRegister(reg, p, DataType.Float);
			}
		}

		/// <summary>Reads the value of the specified motion controller general purpose register.</summary>
		/// <param name="reg">Controller register number to read from.</param>
		/// <param name="regvalue">Variable that will hold the register contents upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetRegister(Int32 reg, ref Double regvalue)
		{
			unsafe 
			{
				fixed (void* p = &regvalue) return GetRegister(reg, p, DataType.Double);
			}
		}

		[DllImport(mcapidll, EntryPoint = "MCGetScale", CharSet = CharSet.Ansi)]
		private static extern Int16 MCGetScale(Int16 hCtlr, Int16 axis, ref Scaling scale);
		/// <summary>Reads the axis current scaling factors.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="scale">aN SCALE structure that will contain scaling information upon
		/// function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetScale(Int16 axis, ref Scaling scale)
		{
			MCGetScale(m_hCtlr, axis, ref scale);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCGetServoOutputPhase", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetServoOutputPhase(Int16 hCtlr, Int16 axis, ref Phase phasing);
		/// <summary>Reads the current servo output phasing for servo axes.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="phasing">One of Phase.Standard or Phase.Reverse upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetServoOutputPhase(Int16 axis, ref Phase phasing)
		{
			return ProcessException(MCGetServoOutputPhase(m_hCtlr, axis, ref phasing));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetStatus", CharSet = CharSet.Ansi)]
		private static extern UInt32 MCGetStatus(Int16 hCtlr, Int16 axis);
		/// <summary>Reads the controller dependent status word for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <returns>The status word.</returns>
		public UInt32 GetStatus(Int16 axis)
		{
			UInt32 status = MCGetStatus(m_hCtlr, axis);
			ProcessException();
			return status;
		}

		[DllImport(mcapidll, EntryPoint = "MCGetStatusEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetStatusEx(Int16 hCtlr, Int16 axis, ref StatusEx status);
		/// <summary>Reads all of the the controller dependent status words for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="status">An STATUSEX data structure that will contain the status 
		/// words upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetStatusEx(Int16 axis, ref StatusEx status)
		{
			return ProcessException(MCGetStatusEx(m_hCtlr, axis, ref status));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetTargetEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetTargetEx(Int16 hCtlr, Int16 axis, ref Double target);
		/// <summary>Reads the move target position, as set by the most recent MoveAbsolute()
		/// or MoveRelative() function call.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="target">Target position, upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetTargetEx(Int16 axis, ref Double target)
		{
			return ProcessException(MCGetTargetEx(m_hCtlr, axis, ref target));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetTorque", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetTorque(Int16 hCtlr, Int16 axis, ref Double torque);
		/// <summary>Reads the axis current torque setting.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="torque">Torque setting upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetTorque(Int16 axis, ref Double torque)
		{
			return ProcessException(MCGetTorque(m_hCtlr, axis, ref torque));
		}

        [DllImport(mcapidll, EntryPoint = "MCGetTrajectoryRate", CharSet = CharSet.Ansi)]
        private static extern Int32 MCGetTrajectoryRate(Int16 hCtlr, ref Int32 rate);
        /// <summary>Reads the current trajectory generator update rate.</summary>
        /// <param name="rate">Rate, one of UpdateRate.Low, UpdateRate.Medium, or UpdateRate.High.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error GetTrajectoryRate(ref UpdateRate rate)
        {
            Int32 temp = 0;
            Int32 ret = MCGetTrajectoryRate(m_hCtlr, ref temp);

            rate = (UpdateRate)temp;
            return ProcessException(ret);
        }

		[DllImport(mcapidll, EntryPoint = "MCGetVectorVelocity", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetVectorVelocity(Int16 hCtlr, Int16 axis, ref Double velocity);
		/// <summary>Reads the current programmed vector velocity for an axis in contouring mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="velocity">Vector velocity upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetVectorVelocity(Int16 axis, ref Double velocity)
		{
			return ProcessException(MCGetVectorVelocity(m_hCtlr, axis, ref velocity));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetVelocityActual", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetVelocityActual(Int16 hCtlr, Int16 axis, ref Double velocity);
		/// <summary>Reads the current actual velocity in whatever units the axis is configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="velocity">Actual velocity upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetVelocityActual(Int16 axis, ref Double velocity)
		{
			return ProcessException(MCGetVelocityActual(m_hCtlr, axis, ref velocity));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetVelocityEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetVelocityEx(Int16 hCtlr, Int16 axis, ref Double velocity);
		/// <summary>Reads the current programmed velocity, in whatever units the axis is 
		/// configured for.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="velocity">Programmed velocity upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetVelocityEx(Int16 axis, ref Double velocity)
		{
			return ProcessException(MCGetVelocityEx(m_hCtlr, axis, ref velocity));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetVelocityOverride", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGetVelocityOverride(Int16 hCtlr, Int16 axis, ref Double over_ride);
		/// <summary>Reads the current velocity override for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="over_ride">Velocity override upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GetVelocityOverride(Int16 axis, ref Double over_ride)
		{
			return ProcessException(MCGetVelocityOverride(m_hCtlr, axis, ref over_ride));
		}

		[DllImport(mcapidll, EntryPoint = "MCGetVersion", CharSet = CharSet.Ansi)]
		private static extern UInt32 MCGetVersion(Int16 hCtlr);
		/// <summary>Reads version information about the API DLL.</summary>
		/// <returns>DLL version information.</returns>
		public UInt32 GetVersion()
		{
			UInt32 version = MCGetVersion(m_hCtlr);
			ProcessException();
			return version;
		}

		[DllImport(mcapidll, EntryPoint = "MCGoEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCGoEx(Int16 hCtlr, Int16 axis, Double param);
		/// <summary>Initiates a motion when operating in velocity mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="param"></param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GoEx(Int16 axis, Double param)
		{
			return ProcessException(MCGoEx(m_hCtlr, axis, param));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCGoHome", CharSet = CharSet.Ansi)]
		private static extern void MCGoHome(Int16 hCtlr, Int16 axis);
		/// <summary>Initiates a home motion for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error GoHome(Int16 axis)
		{
			MCGoHome(m_hCtlr, axis);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCIndexArm", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIndexArm(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Arms the index capture function of a servo axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">The current position of the axis will be set to this position 
		/// when the index signal is sensed.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error IndexArm(Int16 axis, Double position)
		{
			return ProcessException(MCIndexArm(m_hCtlr, axis, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCInterruptOnPosition", CharSet = CharSet.Ansi)]
		private static extern Int32 MCInterruptOnPosition(Int16 hCtlr, Int16 axis, PositionType type, Double position);
		/// <summary>Enables the breakpoint flag of the controller status word.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="type">Position mode, on of PositionType.Absolute or PositionType.Relative.</param>
		/// <param name="position">Absolute or relative interrupt position.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error InterruptOnPosition(Int16 axis, PositionType type, Double position)
		{
			return ProcessException(MCInterruptOnPosition(m_hCtlr, axis, type, position));
		}

		[DllImport(mcapidll, EntryPoint = "MCIsAtTarget", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIsAtTarget(Int16 hCtlr, Int16 axis, Double timeout);
		/// <summary>Waits for the "At Target" condition to go true for the specified axis. Use 
		/// it to determine when motion has completed for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="timeout">Time to wait, in seconds, for the At Target condition to go true.</param>
		/// <returns>TRUE if at target, FALSE if not.</returns>
		public Boolean IsAtTarget(Int16 axis, Double timeout)
		{
			Boolean ret = Convert.ToBoolean(MCIsAtTarget(m_hCtlr, axis, timeout));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "MCIsDigitalFilter", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIsDigitalFilter(Int16 hCtlr, Int16 axis);
		/// <summary>Determine if the digital filter is enabled or not.</summary>
		/// <param name="axis">Axis number.</param>
		/// <returns>TRUE if enabled, FALSE if disabled.</returns>
		public Boolean IsDigitalFilter(Int16 axis)
		{
			Boolean ret = Convert.ToBoolean(MCIsDigitalFilter(m_hCtlr, axis));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "MCIsEdgeFound", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIsEdgeFound(Int16 hCtlr, Int16 axis, Double timeout);
		/// <summary>Waits for the "Edge Found" condition to go true. Use it to determine when 
		/// an open-loop stepper motor homing sequence has detected the edge sensor.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="timeout">Time to wait, in seconds, for the Edge to be found.</param>
		/// <returns>TRUE if edge was found, FALSE if not.</returns>
		public Boolean IsEdgeFound(Int16 axis, Double timeout)
		{
			Boolean ret = Convert.ToBoolean(MCIsEdgeFound(m_hCtlr, axis, timeout));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "MCIsIndexFound", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIsIndexFound(Int16 hCtlr, Int16 axis, Double timeout);
		/// <summary>Waits for the "Index Found" condition to go true. Use it to determine 
		/// when a servo motor homing sequence has detected the encoder index.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="timeout">Time to wait, in seconds, for the Index to be found.</param>
		/// <returns>TRUE if index was found, FALSE if not.</returns>
		public Boolean IsIndexFound(Int16 axis, Double timeout)
		{
			Boolean ret = Convert.ToBoolean(MCIsIndexFound(m_hCtlr, axis, timeout));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "MCIsStopped", CharSet = CharSet.Ansi)]
		private static extern Int32 MCIsStopped(Int16 hCtlr, Int16 axis, Double timeout);
		/// <summary>Waits for the "Trajectory Complete" condition to go true. Use it to 
		/// determine when motion has completed for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="timeout">Time to wait, in seconds, for the Stopped condition to go true.</param>
		/// <returns>TRUE if axis is stopped, FALSE if not.</returns>
		public Boolean IsStopped(Int16 axis, Double timeout)
		{
			Boolean ret = Convert.ToBoolean(MCIsStopped(m_hCtlr, axis, timeout));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "MCLearnPoint", CharSet = CharSet.Ansi)]
		private static extern Int32 MCLearnPoint(Int16 hCtlr, Int16 axis, Int32 index, Learn type);
		/// <summary>Stores the current actual position or target position in point
		/// memory at location specified by the index parameter.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="index">Point memory location to store position information in.</param>
		/// <param name="type">One of Learn.Position or Learn.Target.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error LearnPoint(Int16 axis, Int32 index, Learn type)
		{
			return ProcessException(MCLearnPoint(m_hCtlr, axis, index, type));
		}

		[DllImport(mcapidll, EntryPoint = "MCMacroCall", CharSet = CharSet.Ansi)]
		private static extern void MCMacroCall(Int16 hCtlr, Int16 macro);
		/// <summary>Causes a previously loaded macro to be executed.</summary>
		/// <param name="macro">Macro number.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error MacroCall(Int16 macro)
		{
			MCMacroCall(m_hCtlr, macro);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCMoveAbsolute", CharSet = CharSet.Ansi)]
		private static extern void MCMoveAbsolute(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Initiates an absolute position-mode move.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">Absolute position to move to.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error MoveAbsolute(Int16 axis, Double position)
		{
			MCMoveAbsolute(m_hCtlr, axis, position);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCMoveRelative", CharSet = CharSet.Ansi)]
		private static extern void MCMoveRelative(Int16 hCtlr, Int16 axis, Double distance);
		/// <summary>Initiates an relative position-mode move.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="distance">Distance to move relative to current position.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error MoveRelative(Int16 axis, Double distance)
		{
			MCMoveRelative(m_hCtlr, axis, distance);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCMoveToPoint", CharSet = CharSet.Ansi)]
		private static extern Int32 MCMoveToPoint(Int16 hCtlr, Int16 axis, Int32 index);
		/// <summary>Initiates an absolute move to a position stored in point memory.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="index">Point memory location with position to move to	.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error MoveToPoint(Int16 axis, Int32 index)
		{
			return ProcessException(MCMoveToPoint(m_hCtlr, axis, index));
		}

		[DllImport(mcapidll, EntryPoint = "MCOpen", CharSet = CharSet.Ansi)]
		private static extern Int16 MCOpen(Int16 id, OpenMode mode, String unused);
		/// <summary>Opens a handle to a particular controller for use with subsequent
		/// Motion Control API calls.</summary>
		/// <param name="id">Controller ID number.</param>
		/// <param name="mode">Any of the modes contained in Mcapi.OpenMode.</param>
		/// <param name="unused">Not used.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Open(Int16 id, OpenMode mode, String unused)
		{
			Close();

			Int16 ret;
			if ((ret = MCOpen(id, mode, unused)) > 0)
				m_hCtlr = ret;
			if (ret < 0)
				return ProcessException((Error)(-ret));
			else
				return Mcapi.Error.NoError;
		}

		[DllImport(mcapidll, EntryPoint = "MCReopen", CharSet = CharSet.Ansi)]
		private static extern Int32 MCReopen(Int16 hCtlr, OpenMode newmode);
		/// <summary>Used to change the mode of an existing handle.</summary>
		/// <param name="newmode">Any of the modes contained in Mcapi.OpenMode.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Reopen(OpenMode newmode)
		{
			return ProcessException(MCReopen(m_hCtlr, newmode));
		}

		[DllImport(mcapidll, EntryPoint = "MCRepeat", CharSet = CharSet.Ansi)]
		private static extern Int32 MCRepeat(Int16 hCtlr, Int32 count);
		/// <summary>Inserts a repeat command into a block command - task, 
		/// compound command, or macro.</summary>
		/// <param name="count">Repeat count.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Repeat(Int32 count)
		{
			return ProcessException(MCRepeat(m_hCtlr, count));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCReset", CharSet = CharSet.Ansi)]
		private static extern void MCReset(Int16 hCtlr, Int16 axis);
		/// <summary>Performs a complete reset of the specified axis or controller, leaving 
		/// the specified axis (or axes) in the disabled state.</summary>
		/// <param name="axis">Axis number, set to zero (0) to reset controller.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Reset(Int16 axis)
		{
			MCReset(m_hCtlr, axis);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetAcceleration", CharSet = CharSet.Ansi)]
		private static extern void MCSetAcceleration(Int16 hCtlr, Int16 axis, Double rate);
		/// <summary>Set the programmed acceleration value for the selected axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="rate">New acceleration, specified in the current scaled units for the axis.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetAcceleration(Int16 axis, Double rate)
		{
			MCSetAcceleration(m_hCtlr, axis, rate);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetAnalogEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetAnalogEx(Int16 hCtlr, Int32 channel, UInt32 analogval);
		/// <summary>Set the output level of an analog channel.</summary>
		/// <param name="channel">Analog channel number.</param>
		/// <param name="analogval">New analog channel value.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetAnalogEx(Int32 channel, UInt32 analogval)
		{
			return ProcessException(MCSetAnalogEx(m_hCtlr, channel, analogval));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetAuxEncPos", CharSet = CharSet.Ansi)]
		private static extern void MCSetAuxEncPos(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Set the current position of the auxiliary encoder.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">New auxiliary encoder position.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetAuxEncPos(Int16 axis, Double position)
		{
			MCSetAuxEncPos(m_hCtlr, axis, position);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetCommutation", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetCommutation(Int16 hCtlr, Int16 axis, ref Commutation commutation);
		/// <summary>Sets the commutation settings for the 320 module.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="commutation">An COMMUTATION structure with the new settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetCommutation(Int16 axis, ref Commutation commutation)
		{
			return ProcessException(MCSetCommutation(m_hCtlr, axis, ref commutation));
		}

		[DllImport(mcapidll, EntryPoint = "MCSetContourConfig", CharSet = CharSet.Ansi)]
		private static extern Int16 MCSetContourConfig(Int16 hCtlr, Int16 axis, ref Contour contour);
		/// <summary>Set the contouring configuration.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="contour">A Mcapi.Contour strucuture containing new commutation settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetContourConfig(Int16 axis, ref Contour contour)
		{
			MCSetContourConfig(m_hCtlr, axis, ref contour);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetDeceleration", CharSet = CharSet.Ansi)]
		private static extern void MCSetDeceleration(Int16 hCtlr, Int16 axis, Double rate);
		/// <summary>Set the programmed deceleration for the selected axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="rate">New deceleration, specified in the current scaled units for the axis.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetDeceleration(Int16 axis, Double rate)
		{
			MCSetDeceleration(m_hCtlr, axis, rate);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetDigitalFilter", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetDigitalFilter(Int16 hCtlr, Int16 axis, [In] Double[] coeff, Int32 num);
		/// <summary>Set the digital filter coefficients.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="coeff">Array of coefficients, must be num elements long (or longer). If the 
		/// pointer is NULL the filter will be zeroed (overwriting any previous settings) but no new 
		/// filter values will be stored.</param>
		/// <param name="num">Number of coefficients to retrieve, cannot be larger 
		/// than the maximum digital filter size supported by the controller.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetDigitalFilter(Int16 axis, [In] Double[] coeff, Int32 num)
		{
			return ProcessException(MCSetDigitalFilter(m_hCtlr, axis, coeff, num));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetFilterConfigEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetFilterConfigEx(Int16 hCtlr, Int16 axis, ref FilterEx filter);
		/// <summary>Configures the PID loop settings for a servo motor or the closed-loop settings 
		/// settings for a stepper motor operating in closed-loop mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="filter">A Mcapi.FilterEx structure containing the new settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetFilterConfigEx(Int16 axis, ref FilterEx filter)
		{
			return ProcessException(MCSetFilterConfigEx(m_hCtlr, axis, ref filter));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetGain", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetGain(Int16 hCtlr, Int16 axis, Double gain);
		/// <summary>Set the proportional gain of a servo's feedback loop.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="gain">New gain setting.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetGain(Int16 axis, Double gain)
		{
			return ProcessException(MCSetGain(m_hCtlr, axis, gain));
		}

		[DllImport(mcapidll, EntryPoint = "MCSetJogConfig", CharSet = CharSet.Ansi)]
		private static extern Int16 MCSetJogConfig(Int16 hCtlr, Int16 axis, ref Jog jog);
		/// <summary>Set the jogging mode configuration.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="jog">A JOG structure containing jog settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetJogConfig(Int16 axis, ref Jog jog)
		{
			MCSetJogConfig(m_hCtlr, axis, ref jog);
			return ProcessException();
		}

        [DllImport(mcapidll, EntryPoint = "MCSetJogConfigEx", CharSet = CharSet.Ansi)]
        private static extern Int32 MCSetJogConfigEx(Int16 hCtlr, Int16 axis, ref JogEx jog);
        /// <summary>Set the jogging mode configuration.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="jog">A JOGEX structure containing jog settings.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error SetJogConfigEx(Int16 axis, ref JogEx jog)
        {
            return ProcessException(MCSetJogConfigEx(m_hCtlr, axis, ref jog));
        }

		[DllImport(mcapidll, EntryPoint = "MCSetLimits", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetLimits(Int16 hCtlr, Int16 axis, Int16 hardmode, Int16 softmode, Double softlimitminus, Double softlimitplus);
		/// <summary>Set the current hard and soft limit settings.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="hardmode">One or more of LimitMode.Off, LimitMode.Plus, LimitMode.Minus, 
		/// LimitMode.Both, LimitMode.Abrupt LimitMode.Smooth and LimitMode.Invert, OR'ed together.</param>
		/// <param name="softmode">See settings for hardlimitmode.</param>
		/// <param name="softlimitminus">Negative limit value for soft limits.</param>
		/// <param name="softlimitplus">Positive limit value for soft limits.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetLimits(Int16 axis, LimitMode hardmode, LimitMode softmode, Double softlimitminus, Double softlimitplus)
		{
			return ProcessException(MCSetLimits(m_hCtlr, axis, Convert.ToInt16(hardmode), Convert.ToInt16(softmode), softlimitminus, softlimitplus));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetModuleInputMode", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetModuleInputMode(Int16 hCtlr, Int16 axis, InputMode mode);
		/// <summary>Set the axis input mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of the InputMode constants.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetModuleInputMode(Int16 axis, InputMode mode)
		{
			return ProcessException(MCSetModuleInputMode(m_hCtlr, axis, mode));
		}

		[DllImport(mcapidll, EntryPoint = "MCSetModuleOutputMode", CharSet = CharSet.Ansi)]
		private static extern void MCSetModuleOutputMode(Int16 hCtlr, Int16 axis, Int16 mode);
		/// <summary>Set the axis output mode.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of OutputMode.Bipolar, OutputMode.Unipolar, OutputMode.BipolarPWM,
		/// OutputMode.UnipolarPWM, OutputMode.PulseAndDirection, or OutputMode.ClockwiseCounterClockwise.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetModuleOutputMode(Int16 axis, OutputMode mode)
		{
			MCSetModuleOutputMode(m_hCtlr, axis, Convert.ToInt16(mode));
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetMotionConfigEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetMotionConfigEx(Int16 hCtlr, Int16 axis, ref MotionEx motion);
		/// <summary>Configure the motion parameters for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="motion">A MCapi.MotionEx structure containing the new motion parameters.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetMotionConfigEx(Int16 axis, ref MotionEx motion)
		{
			return ProcessException(MCSetMotionConfigEx(m_hCtlr, axis, ref motion));
		}

		[DllImport(mcapidll, EntryPoint = "MCSetOperatingMode", CharSet = CharSet.Ansi)]
		private static extern void MCSetOperatingMode(Int16 hCtlr, Int16 axis, Int16 caxis, UInt16 mode);
		/// <summary>Set the controller operating mode (position mode, velocity mode, etc.).</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="caxis">Contouring master axis (used for contour mode only).</param>
		/// <param name="mode">One of the OperatingMode constants.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetOperatingMode(Int16 axis, Int16 caxis, OperatingMode mode)
		{
			MCSetOperatingMode(m_hCtlr, axis, caxis, Convert.ToUInt16(mode));
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetPosition", CharSet = CharSet.Ansi)]
		private static extern void MCSetPosition(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Sets the current position for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">New position.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetPosition(Int16 axis, Double position)
		{
			MCSetPosition(m_hCtlr, axis, position);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetProfile", CharSet = CharSet.Ansi)]
		private static extern void MCSetProfile(Int16 hCtlr, Int16 axis, ProfileType mode);
		/// <summary>Set the acceleration/deceleration profile for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="mode">One of ProfileType.Trapezoidal, ProfileType.SCurve, or ProfileType.Parabolic.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetProfile(Int16 axis, ProfileType mode)
		{
			MCSetProfile(m_hCtlr, axis, mode);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetRegister", CharSet = CharSet.Ansi)]
		private static unsafe extern Int32 MCSetRegister(Int16 hCtlr, Int32 reg, void* regvalue, DataType type);
		/// <summary>Set the value of the specified general purpose motion controller register.</summary>
		/// <param name="reg">Register number.</param>
		/// <param name="regvalue">New register value.</param>
		/// <param name="type">Data type, one of the DataType enum values. Must match the type of the regvalue variable.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public unsafe Error SetRegister(Int32 reg, void* regvalue, DataType type)
		{
			return ProcessException(MCSetRegister(m_hCtlr, reg, regvalue, type));
		}

		/// <summary>Set the value of the specified general purpose motion controller register.</summary>
		/// <param name="reg">Register number.</param>
		/// <param name="regvalue">New register value.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetRegister(Int32 reg, Int32 regvalue)
		{
			unsafe 
			{
				return SetRegister(reg, &regvalue, DataType.Long);
			}
		}

		/// <summary>Set the value of the specified general purpose motion controller register.</summary>
		/// <param name="reg">Register number.</param>
		/// <param name="regvalue">New register value.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetRegister(Int32 reg, Single regvalue)
		{
			unsafe 
			{
				return SetRegister(reg, &regvalue, DataType.Float);
			}
		}

		/// <summary>Set the value of the specified general purpose motion controller register.</summary>
		/// <param name="reg">Register number.</param>
		/// <param name="regvalue">New register value.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetRegister(Int32 reg, Double regvalue)
		{
			unsafe 
			{
				return SetRegister(reg, &regvalue, DataType.Double);
			}
		}

		[DllImport(mcapidll, EntryPoint = "MCSetScale", CharSet = CharSet.Ansi)]
		private static extern Int16 MCSetScale(Int16 hCtlr, Int16 axis, ref Scaling scale);
		/// <summary>Sets scaling for the specified axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="scale">A Mcapi.Scaling structure containing new scaling settings.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetScale(Int16 axis, ref Scaling scale)
		{
			MCSetScale(m_hCtlr, axis, ref scale);
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetServoOutputPhase", CharSet = CharSet.Ansi)]
		private static extern void MCSetServoOutputPhase(Int16 hCtlr, Int16 axis, Phase phasing);
		/// <summary>Set the output phasing for the specified servo axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="phasing">One of Phase.Standard or Phase.Reverse.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetServoOutputPhase(Int16 axis, Phase phasing)
		{
			MCSetServoOutputPhase(m_hCtlr, axis, phasing);
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetTimeoutEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetTimeoutEx(Int16 hCtlr, Double timeout, ref Double oldtimeout);
		/// <summary>Set the timeout period for API functions.</summary>
		/// <param name="timeout">New timeout value, in seconds.</param>
		/// <param name="oldtimeout">Previous timeout upon function return.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetTimeoutEx(Double timeout, ref Double oldtimeout)
		{
			return ProcessException(MCSetTimeoutEx(m_hCtlr, timeout, ref oldtimeout));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetTorque", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetTorque(Int16 hCtlr, Int16 axis, Double torque);
		/// <summary>Set the maximum output level for servos.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="torque">New torque, in volts.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetTorque(Int16 axis, Double torque)
		{
			return ProcessException(MCSetTorque(m_hCtlr, axis, torque));
		}

        [DllImport(mcapidll, EntryPoint = "MCSetTrajectoryRate", CharSet = CharSet.Ansi)]
        private static extern Int32 MCSetTrajectoryRate(Int16 hCtlr, Int32 rate);
        /// <summary>Sets the trajectopry update rate.</summary>
        /// <param name="rate">New rate, one of UpdateRate.Low, UpdateRate.Medium, or UpdateRate.High.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error SetTrajectoryRate(UpdateRate rate)
        {
            return ProcessException(MCSetTrajectoryRate(m_hCtlr, Convert.ToInt32(rate)));
        }

		[DllImport(mcapidll, EntryPoint = "MCSetVectorVelocity", CharSet = CharSet.Ansi)]
		private static extern Int32 MCSetVectorVelocity(Int16 hCtlr, Int16 axis, Double velocity);
		/// <summary>Sets the vector velocity for the specified contouring axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="velocity">New vector velocity, in whatever units the axis is configured for.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetVectorVelocity(Int16 axis, Double velocity)
		{
			return ProcessException(MCSetVectorVelocity(m_hCtlr, axis, velocity));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCSetVelocity", CharSet = CharSet.Ansi)]
		private static extern void MCSetVelocity(Int16 hCtlr, Int16 axis, Double velocity);
		/// <summary>Set the programmed velocity for the selected axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="velocity">New programmed velocity, in whatever units the axis is configured for.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetVelocity(Int16 axis, Double velocity)
		{
			MCSetVelocity(m_hCtlr, axis, velocity);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCSetVelocityOverride", CharSet = CharSet.Ansi)]
		private static extern void MCSetVelocityOverride(Int16 hCtlr, Int16 axis, Double rate);
		/// <summary>Set the velocity override factor for an axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="rate">New velocity override.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error SetVelocityOverride(Int16 axis, Double rate)
		{
			MCSetVelocityOverride(m_hCtlr, axis, rate);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCStop", CharSet = CharSet.Ansi)]
		private static extern void MCStop(Int16 hCtlr, Int16 axis);
		/// <summary>Stop the specified axis or axes using the pre-programmed deceleration values.</summary>
		/// <param name="axis">Axis number or zero (0) for all axes.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error Stop(Int16 axis)
		{
			MCStop(m_hCtlr, axis);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "MCTranslateErrorEx", CharSet = CharSet.Ansi)]
		private static extern Int32 MCTranslateErrorEx(Int16 error, StringBuilder buffer, Int32 size);
		/// <summary>Translate numeric error codes into ASCII text messages.</summary>
		/// <param name="error">Error number.</param>
		/// <param name="buffer">String buffer.</param>
		/// <param name="size">Size of string buffer.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error TranslateErrorEx(Error error, StringBuilder buffer, Int32 size)
		{
			return ProcessException(MCTranslateErrorEx(Convert.ToInt16(error), buffer, size));
		}

		/// <summary>
		/// Causes the motion controller to wait the specified number of seconds before 
		/// executing any additional API function calls.
		/// </summary>
		/// <param name="hCtlr">Motion controller handle.</param>
		/// <param name="period">Wait period, in seconds.</param>
		[DllImport(mcapidll, EntryPoint = "MCWait", CharSet = CharSet.Ansi)]
		public static extern void Wait(Int16 hCtlr, Double period);
		
		[DllImport(mcapidll, EntryPoint = "MCWaitForDigitalIO", CharSet = CharSet.Ansi)]
		private static extern void MCWaitForDigitalIO(Int16 hCtlr, Int16 channel, Int16 state);
		/// <summary>
		/// Waits for the specified digital I/O channel to go on or off.
		/// </summary>
		/// <param name="hCtlr">Motion controller handle.</param>
		/// <param name="channel">Digital I/O channel number.</param>
		/// <param name="state">TRUE to wait for channel to go, FALSE to wait for it to go off.</param>
		public static void WaitForDigitalIO(Int16 hCtlr, Int16 channel, Boolean state)
		{
			MCWaitForDigitalIO(hCtlr, channel, Convert.ToInt16(state));
		}

		[DllImport(mcapidll, EntryPoint = "MCWaitForEdge", CharSet = CharSet.Ansi)]
		private static extern Int32 MCWaitForEdge(Int16 hCtlr, Int16 axis, Int16 state);
		/// <summary>Wait for the coarse home input to go to the specified logic level.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="state">TRUE to wait for it to go positive, FALSE to wait
		/// for it to go negative.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForEdge(Int16 axis, Boolean state)
		{
			return ProcessException(MCWaitForEdge(m_hCtlr, axis, Convert.ToInt16(state)));
		}

		[DllImport(mcapidll, EntryPoint = "MCWaitForIndex", CharSet = CharSet.Ansi)]
		private static extern Int32 MCWaitForIndex(Int16 hCtlr, Int16 axis);
		/// <summary>Wait until the index pulse has been observed on a servo axis.</summary>
		/// <param name="axis">Axis number.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForIndex(Int16 axis)
		{
			return ProcessException(MCWaitForIndex(m_hCtlr, axis));
		}
		
		[DllImport(mcapidll, EntryPoint = "MCWaitForPosition", CharSet = CharSet.Ansi)]
		private static extern void MCWaitForPosition(Int16 hCtlr, Int16 axis, Double position);
		/// <summary>Wait for an axis to reach the specified position before allowing the 
		/// next command to execute.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="position">Absolute position to wait for</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForPosition(Int16 axis, Double position)
		{
			MCWaitForPosition(m_hCtlr, axis, position);
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCWaitForRelative", CharSet = CharSet.Ansi)]
		private static extern void MCWaitForRelative(Int16 hCtlr, Int16 axis, Double distance);
		/// <summary>Waits for an axis to reach a position that is specified relative to the 
		/// target position.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="distance">Distance to wait for.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForRelative(Int16 axis, Double distance)
		{
			MCWaitForRelative(m_hCtlr, axis, distance);
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCWaitForStop", CharSet = CharSet.Ansi)]
		private static extern void MCWaitForStop(Int16 hCtlr, Int16 axis, Double period);
		/// <summary>Wait for the specified Axis or all axes to come to a stop.</summary>
		/// <param name="axis">Axis number or zero (0) for all axes.</param>
		/// <param name="period">Dwell time to wait after the axis has stopped, in seconds.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForStop(Int16 axis, Double period)
		{
			MCWaitForStop(m_hCtlr, axis, period);
			return ProcessException();
		}
		
		[DllImport(mcapidll, EntryPoint = "MCWaitForTarget", CharSet = CharSet.Ansi)]
		private static extern void MCWaitForTarget(Int16 hCtlr, Int16 axis, Double period);
		/// <summary>Waits for an axis to reach its target position.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="period">Dwell time to wait after the axis has reached the target, in seconds.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error WaitForTarget(Int16 axis, Double period)
		{
			MCWaitForTarget(m_hCtlr, axis, period);
			return ProcessException();
		}

		[DllImport(mcapidll, EntryPoint = "pmccmdex", CharSet = CharSet.Ansi)]
		private static unsafe extern Int32 _pmccmdex(Int16 hCtlr, Int16 axis, Int16 command, void* argument, DataType type);
		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <param name="argument">Command argument.</param>
		/// <param name="type">Type of command argument, one of the DataType enumeration values.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public unsafe Error pmccmdex(Int16 axis, Int16 command, void* argument, DataType type)
		{
			unsafe 
			{
				return ProcessException(_pmccmdex(m_hCtlr, axis, command, argument, type));
			}
		}

		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmccmdex(Int16 axis, Int16 command)
		{
			unsafe 
			{
				return ProcessException(_pmccmdex(m_hCtlr, axis, command, null, DataType.None));
			}
		}

		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <param name="argument">Command argument.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmccmdex(Int16 axis, Int16 command, Int32 argument)
		{
			unsafe 
			{
				return ProcessException(_pmccmdex(m_hCtlr, axis, command, &argument, DataType.Long));
			}
		}

		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <param name="argument">Command argument.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmccmdex(Int16 axis, Int16 command, Single argument)
		{
			unsafe 
			{
				return ProcessException(_pmccmdex(m_hCtlr, axis, command, &argument, DataType.Float));
			}
		}

		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <param name="argument">Command argument.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmccmdex(Int16 axis, Int16 command, Double argument)
		{
			unsafe 
			{
				return ProcessException(_pmccmdex(m_hCtlr, axis, command, &argument, DataType.Double));
			}
		}

		/// <summary>Format an MCCL command and issue to the controller.</summary>
		/// <param name="axis">Axis number.</param>
		/// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
		/// for your motion controller.</param>
		/// <param name="argument">Command argument.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmccmdex(Int16 axis, Int16 command, String argument)
		{
			unsafe 
			{
				fixed (void* p = argument.ToCharArray()) return ProcessException(_pmccmdex(m_hCtlr, axis, command, p, DataType.String));
			}
		}

        [DllImport(mcapidll, EntryPoint = "pmccmdrpyex", CharSet = CharSet.Ansi)]
        private static unsafe extern Int32 _pmccmdrpyex(Int16 hCtlr, Int16 axis, Int16 command, void* arg, DataType arg_type, void* rpy, DataType rpy_type);
        /// <summary>Format an MCCL command and issue to the controller, read reply.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
        /// for your motion controller.</param>
        /// <param name="arg">Command argument.</param>
        /// <param name="arg_type">Type of command argument, one of the DataType enumeration values.</param>
        /// <param name="rpy">Reply argument.</param>
        /// <param name="rpy_type">Type of reply argument, one of the DataType enumeration values.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public unsafe Error pmccmdrpyex(Int16 axis, Int16 command, void* arg, DataType arg_type, void* rpy, DataType rpy_type)
        {
            unsafe
            {
                return ProcessException(_pmccmdrpyex(m_hCtlr, axis, command, arg, arg_type, rpy, rpy_type));
            }
        }

        /// <summary>Format an MCCL command and issue to the controller.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
        /// for your motion controller.</param>
        /// <param name="arg">Argument to command to send.</param>
        /// <param name="rpy">Variable to receive reply value.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error pmccmdrpyex(Int16 axis, Int16 command, Int32 arg, Int32 rpy)
        {
            unsafe
            {
                return ProcessException(_pmccmdrpyex(m_hCtlr, axis, command, &arg, DataType.Long, &rpy, DataType.Long));
            }
        }

        /// <summary>Format an MCCL command and issue to the controller.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
        /// for your motion controller.</param>
        /// <param name="arg">Argument to command to send.</param>
        /// <param name="rpy">Variable to receive reply value.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error pmccmdrpyex(Int16 axis, Int16 command, Int32 arg, Double rpy)
        {
            unsafe
            {
                return ProcessException(_pmccmdrpyex(m_hCtlr, axis, command, &arg, DataType.Long, &rpy, DataType.Double));
            }
        }

        /// <summary>Format an MCCL command and issue to the controller.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
        /// for your motion controller.</param>
        /// <param name="arg">Argument to command to send.</param>
        /// <param name="rpy">Variable to receive reply value.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error pmccmdrpyex(Int16 axis, Int16 command, Double arg, Int32 rpy)
        {
            unsafe
            {
                return ProcessException(_pmccmdrpyex(m_hCtlr, axis, command, &arg, DataType.Double, &rpy, DataType.Long));
            }
        }

        /// <summary>Format an MCCL command and issue to the controller.</summary>
        /// <param name="axis">Axis number.</param>
        /// <param name="command">MCCL command to execute - see MCCL.H and the User's Manual 
        /// for your motion controller.</param>
        /// <param name="arg">Argument to command to send.</param>
        /// <param name="rpy">Variable to receive reply value.</param>
        /// <returns>An Mcapi.Error error status.</returns>
        public Error pmccmdrpyex(Int16 axis, Int16 command, Double arg, Double rpy)
        {
            unsafe
            {
                return ProcessException(_pmccmdrpyex(m_hCtlr, axis, command, &arg, DataType.Double, &rpy, DataType.Double));
            }
        }

		[DllImport(mcapidll, EntryPoint = "pmcgetc", CharSet = CharSet.Ansi)]
		private static extern Int16 _pmcgetc(Int16 hCtlr);
		/// <summary>Read a single character from the controller ASCII interface.</summary>
		/// <returns>The character or zero (0) if no characters are waiting.</returns>
		public Int16 pmcgetc()
		{
			Int16 ret = _pmcgetc(m_hCtlr);
			ProcessException();
			return ret;
		}
		
		[DllImport(mcapidll, EntryPoint = "pmcgetramex", CharSet = CharSet.Ansi)]
		private static extern Int32 _pmcgetramex(Int16 hCtlr, UInt32 offset, [In, Out] Byte[] buffer, UInt32 size);
		/// <summary>Read directly from controller memory.</summary>
		/// <param name="offset">Base address to begin reading from.</param>
		/// <param name="buffer">Buffer to hold data readback from controller.</param>
		/// <param name="size">Size of buffer, in bytes.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmcgetramex(UInt32 offset, [In, Out] Byte[] buffer, UInt32 size)
		{
			return ProcessException(_pmcgetramex(m_hCtlr, offset, buffer, size));
		}

		[DllImport(mcapidll, EntryPoint = "pmcgets", CharSet = CharSet.Ansi)]
		private static extern Int16 _pmcgets(Int16 hCtlr, StringBuilder buffer, Int16 size);
		/// <summary>Read a null-terminated ASCII string from the controller ASCII interface.</summary>
		/// <param name="buffer">Buffer to store read back string.</param>
		/// <param name="size">Size of buffer.</param>
		/// <returns>Number of characters read.</returns>
		public Int16 pmcgets(StringBuilder buffer, Int16 size)
		{
			Int16 ret = _pmcgets(m_hCtlr, buffer, size);
			ProcessException();
			return ret;
		}

        [DllImport(mcapidll, EntryPoint = "pmclock", CharSet = CharSet.Ansi)]
        private static extern UInt32 _pmclock(Int16 hCtlr, UInt32 wait_msec);
        /// <summary>Manually signal the controller semaphore.</summary>
        /// <remarks>
        /// <para>This call allows applications that need to send multiple commands without being
        /// interrupted by another thread to do so. Caller's are responsible for calling pmcunlock()
        /// when finished or the interface will hang. Standard MCAPI calls do not require the 
        /// application to manage the semaphores.</para>
        /// </remarks>
        /// <param name="wait_msec">Time to wait for the semaphore in milliseconds, use -1 or 
        /// the WIndows constant INFINITE to wait as long as is needed.</param>
        /// <returns>One of the Windows statuses from the Windows call WaitForSingleObject(), 0x00000080
        /// if the wait was abandoned (WAIT_ABANDONED), 0x00000000 if the object was signaled i.e. success
        /// (WAIT_OBJECT_0), 0x00000102 for timeout (WAIT_TIMEOUT), or 0xFFFFFFFF for failure 
        /// (WAIT_FAILED).</returns>
        public UInt32 pmclock(UInt32 wait_msec)
        {
            return _pmclock(m_hCtlr, wait_msec);
        }

        //pmclookupvar)(HCTRLR hCtlr, const char* varname, DWORD* address);
        [DllImport(mcapidll, EntryPoint = "pmclookupvar", CharSet = CharSet.Ansi)]
        private static unsafe extern Int32 _pmclookupvar(Int16 hCtlr, String varname, ref UInt32 address);
        /// <summary>Lookup the address of a controller variable.</summary>
        /// <remarks>
        /// <para>This function will query the controller for address of a motor variable
        /// and return that address (the address may then be used with pmccmdrpyex() and
        /// one of the read commands (RD, RL, RS) to read the variable.</para>
        /// </remarks>
        /// <param name="varname">Name of the varaible to lookup. For a list of variable names (and their types)
        /// issue an "HE1" command in WinControl.</param>
        /// <param name="address">On return the address of the variable, or -1 if the variable was
        /// not found.</param>
        /// <returns>MCERR_NOERROR (zero) is returned if the function completed successfully, if it 
        /// did not complete successfully one of the MCERR_xxxx error codes is returned.</returns>
        public Error pmclookupvar(String varname, ref UInt32 address)
        {
            unsafe
            {
                return ProcessException(_pmclookupvar(m_hCtlr, varname, ref address));
            }
        }
		
		[DllImport(mcapidll, EntryPoint = "pmcputc", CharSet = CharSet.Ansi)]
		private static extern Int16 _pmcputc(Int16 hCtlr, Int16 character);
		/// <summary>Write a single character to the controller ASCII interface.</summary>
		/// <param name="character">Character to write.</param>
		/// <returns>One (1) if the character was written, zero (0) if not.</returns>
		public Int16 pmcputc(Int16 character)
		{
			Int16 ret = _pmcputc(m_hCtlr, character);
			ProcessException();
			return ret;
		}
		
		[DllImport(mcapidll, EntryPoint = "pmcputramex", CharSet = CharSet.Ansi)]
		private static extern Int32 _pmcputramex(Int16 hCtlr, UInt32 offset, [In] Byte[] buffer, UInt32 size);
		/// <summary>Write data directly into the controller's memory.</summary>
		/// <param name="offset">Base address to begin writing at.</param>
		/// <param name="buffer">Buffer of data to write.</param>
		/// <param name="size">Size of buffer (amount of data to write), in bytes.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmcputramex(UInt32 offset, [In] Byte[] buffer, UInt32 size)
		{
			return ProcessException(_pmcputramex(m_hCtlr, offset, buffer, size));
		}

		[DllImport(mcapidll, EntryPoint = "pmcputs", CharSet = CharSet.Ansi)]
		private static extern Int16 _pmcputs(Int16 hCtlr, String buffer);
		/// <summary>Write a NULL terminated command string to the controller ASCII interface.</summary>
		/// <param name="buffer">String to write.</param>
		/// <returns>Number of characters written.</returns>
		public Int16 pmcputs(String buffer)
		{
			Int16 ret = _pmcputs(m_hCtlr, buffer);
			ProcessException();
			return ret;
		}
	
		[DllImport(mcapidll, EntryPoint = "pmcrdy", CharSet = CharSet.Ansi)]
		private static extern Int16 _pmcrdy(Int16 hCtlr);
		/// <summary>Check to see if the specified controller to see if it is ready to 
		/// accept a binary command buffer.</summary>
		/// <returns>TRUE if ready for a binary command, FALSE if not.</returns>
		public Boolean pmcrdy()
		{
			Boolean ret = Convert.ToBoolean(_pmcrdy(m_hCtlr));
			ProcessException();
			return ret;
		}

		[DllImport(mcapidll, EntryPoint = "pmcrpyex", CharSet = CharSet.Ansi)]
		private static unsafe extern Int32 _pmcrpyex(Int16 hCtlr, void* reply, DataType type);
		/// <summary>Reads a reply value from the binary interface of a controller.</summary>
		/// <param name="reply">Variable to hold reply.</param>
		/// <param name="type">Reply type, one of the DataType enumeration values.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public unsafe Error pmcrpyex(void* reply, DataType type)
		{
			unsafe 
			{
				return ProcessException(_pmcrpyex(m_hCtlr, reply, type));
			}
		}

		/// <summary>Reads a reply value from the binary interface of a controller.</summary>
		/// <param name="reply">Variable to hold reply.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmcrpyex(ref Int32 reply)
		{
			unsafe 
			{
				fixed (void* p = &reply) return ProcessException(_pmcrpyex(m_hCtlr, p, DataType.Long));
			}
		}

		/// <summary>Reads a reply value from the binary interface of a controller.</summary>
		/// <param name="reply">Variable to hold reply.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmcrpyex(ref Single reply)
		{
			unsafe 
			{
				fixed (void* p = &reply) return ProcessException(_pmcrpyex(m_hCtlr, p, DataType.Float));
			}
		}

		/// <summary>Reads a reply value from the binary interface of a controller.</summary>
		/// <param name="reply">Variable to hold reply.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public Error pmcrpyex(ref Double reply)
		{
			unsafe 
			{
				fixed (void* p = &reply) return ProcessException(_pmcrpyex(m_hCtlr, p, DataType.Double));
			}
		}

        [DllImport(mcapidll, EntryPoint = "pmcunlock", CharSet = CharSet.Ansi)]
        private static extern UInt32 _pmcunlock(Int16 hCtlr);
        /// <summary>Manually un-signal the controller semaphore.</summary>
        /// <remarks>
        /// <para>This call allows applications that need to send multiple commands without being
        /// interrupted by another thread to do so. Caller's are responsible for calling pmcunlock() 
        /// when finished or the interface will hang. Standard MCAPI calls do not require the 
        /// application to manage the semaphores.</para>
        /// </remarks>
        public void pmcunlock()
        {
            _pmcunlock(m_hCtlr);
        }

		//
		// -- Helper Function declarations -------------------------------------
		//

		private Error ProcessException(Error error)
		{
            if (m_throw_exceptions && (error != Mcapi.Error.NoError) && (error != Mcapi.Error.NotSupported))
			{
				StringBuilder buffer = new StringBuilder(128);
				TranslateErrorEx(error, buffer, buffer.Capacity);
				throw new McapiException(error, buffer.ToString());
			}
			return error;
		}

		private Error ProcessException(Int32 error)
		{
			return ProcessException((Error)error);
		}

		private Error ProcessException()
		{
			return ProcessException(GetError());
		}
	}
}
