//
//  mcdlg.cs - class encapsulization for MCDLG function library.
//
//  $Id: Mcdlg.cs 856 2015-01-05 17:55:13Z brian $
//
//  Copyright (c) 2006-2015 by Precision Micro Control Corp. All rights reserved.
//
using System;
using System.Text;
using System.Runtime.InteropServices;

namespace McapiNet
{
	/// <summary>
	/// Summary description for Mcdlg.
	/// </summary>
	public class Mcdlg
	{
		/// <summary>
		/// Motion Dialogs class constructor.
		/// </summary>
		public Mcdlg()
		{
		}

        //
        // -- Data -------------------------------------------------------------
        //
#if MCAPI_ISX64
        private const string mcdlgdll = "mcdlg64.dll";          // 64-bit build uses 64-bit MCDLG
#else
        private const string mcdlgdll = "mcdlg32.dll";          // 32-bit build uses 32-bit MCDLG
#endif

		/// <summary>Common flags for Mcdlg functions.</summary>
		[Flags()]
			public enum Flags : int
		{
			/// <summary>No flags set.</summary>
			None = 0,
			/// <summary>Restrict response to the name only for MCDLG_ControllerDesc() / MCDLG_ModuleDesc().</summary>
			NameOnly = 1,
			/// <summary>Restrict response to the description only for MCDLG_ControllerDesc() / MCDLG_ModuleDesc().</summary>
			DescOnly = 2,
			/// <summary>Show warning prompt if axis is active.</summary>
			Prompt = 1,
			/// <summary>Do not save/restore motion related parameters.</summary>
			NoMotion = 2,
			/// <summary>Do not save/restore pid filter related parameters.</summary>
			NoFilter = 4,
			/// <summary>Do not save/restore servo phase parameters.</summary>
			NoPhase = 8,
			/// <summary>Do not save/restore position.</summary>
			NoPosition = 16,
			/// <summary>Do not save/restore scaling parameters.</summary>
			NoScale = 32,
			/// <summary>Do not save/restore hard limit related parameters.</summary>
			NoHardLimits = 64,
			/// <summary>Do not save/restore soft limit related parameters.</summary>
			NoSoftLimits = 128,
			/// <summary>Do not save/restore loop update/step rate related parameters.</summary>
			NoRates = 256,
			/// <summary>Do not save/restore accel/decel profile related parameters.</summary>
			NoProfiles = 512,
			/// <summary>Do not save/restore miscellaneous parameters.</summary>
			NoMisc = 1024,
			/// <summary>Check if axis is active before saving or restoring.</summary>
			CheckActive = 2048,
			/// <summary>Do not save/restore output mode related parameters.</summary>
			NoOutputMode = 4096
		}

		/// <summary>Display a simple About dialog box that includes version information
		/// about both the application and the Motion Control API.</summary>
		/// <param name="hwnd">Handle to parent window or NULL.</param>
		/// <param name="title">An optional title string for the About dialog box. If this pointer 
		/// is NULL or points to a zero length string the default title of About is used.</param>
		/// <param name="bitmap_id">An optional Bitmap resource identifier.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_AboutBox", CharSet = CharSet.Ansi)]
		public static extern Mcapi.Error AboutBox(IntPtr hwnd, String title, Int32 bitmap_id);

		/// <summary>Return the file extension for MCCL command files for a particular
		/// motion controller type.</summary>
		/// <param name="type">Motion controller type.</param>
		/// <param name="flags">Reserved for future use (set to Flags.None).</param>
		/// <param name="buffer">String buffer that will hold the file extension.</param>
		/// <param name="size">Size of buffer, in bytes.</param>
		/// <returns>A pointer to the returnb string.</returns>
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_CommandFileExt", CharSet = CharSet.Ansi)]
		public static extern IntPtr CommandFileExt(Int32 type, Flags flags, StringBuilder buffer, Int32 size);

		[DllImport(mcdlgdll, EntryPoint = "MCDLG_ConfigureAxis", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_ConfigureAxis(IntPtr hwnd, Int16 hCtlr, Int16 axis, Flags flags, String title);
		/// <summary>Display a servo or stepper axis setup dialog that permits user 
		/// configuration of the axis.</summary>
		/// <param name="hwnd">Handle to parent window or NULL.</param>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="axis">Axis number.</param>
		/// <param name="flags"> Optionally one or more of of Mcdlg.CheckActive and Mcdlg.Prompt,
		/// OR'ed together.</param>
		/// <param name="title">An optional title string for the axis. If this pointer is NULL or 
		/// points to a zero length string the default title, which includes the axis number and 
		/// a description of the axis type is used.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error ConfigureAxis(IntPtr hwnd, Mcapi controller, Int16 axis, Flags flags, String title)
		{
			return ProcessException(MCDLG_ConfigureAxis(hwnd, controller.Handle, axis, flags, title));
		}
		
		/// <summary>Return a descriptive string for the specified motion controller type.</summary>
		/// <param name="type">Motion Controller type.</param>
		/// <param name="flags">Optionally one of Mcdlg.NameOnly or Mcdlg.DescOnly.</param>
		/// <param name="buffer">string buffer that will hold the descriptive string.</param>
		/// <param name="size">Buffer size.</param>
		/// <returns>Pointer to the string buffer.</returns>
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_ControllerDescEx", CharSet = CharSet.Ansi)]
		public static extern IntPtr ControllerDescEx(Mcapi.ControllerType type, Flags flags, StringBuilder buffer, Int32 size);
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_ControllerInfo", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_ControllerInfo(IntPtr hwnd, Int16 hCtlr, Flags flags, String title);
		/// <summary>Display configuration information about the specified motion controller.</summary>
		/// <param name="hwnd">Handle to parent window or NULL.</param>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="flags">Reserved for future use (set to Flags.None).</param>
		/// <param name="title">An optional title string for the dialog box. If this argument 
		/// is NULL or points to a zero length string the default title is used.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error ControllerInfo(IntPtr hwnd, Mcapi controller, Flags flags, String title)
		{
			return ProcessException(MCDLG_ControllerInfo(hwnd, controller.Handle, flags, title));
		}
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_DownloadFile", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_DownloadFile(IntPtr hwnd, Int16 hCtlr, Flags flags, String filename);
		/// <summary>Download an ASCII command file to the specified motion controller.</summary>
		/// <param name="hwnd">Handle to parent window or NULL.</param>
		/// <param name="controller">Motion controller object (must be open in ASCII mode).</param>
		/// <param name="flags">Reserved for future use (set to Flags.None).</param>
		/// <param name="filename">Path / filename of file to download.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error DownloadFile(IntPtr hwnd, Mcapi controller, Flags flags, String filename)
		{
			return ProcessException(MCDLG_DownloadFile(hwnd, controller.Handle, flags, filename));
		}

		//  not used publicly
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_Initialize", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_Initialize();

		[DllImport(mcdlgdll, EntryPoint = "MCDLG_ListControllers", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_ListControllers([In, Out] Int16[] idarray, Int16 size);
		/// <summary>Enumerate the types of motion controllers installed.</summary>
		/// <param name="idarray">Array of controller types upon function return.</param>
		/// <param name="size">Size of array (number of elements, not bytes).</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error ListControllers([In, Out] Int16[] idarray, Int16 size)
		{
			return ProcessException(MCDLG_ListControllers(idarray, size));
		}
		
		/// <summary>Return a descriptive string for the specified module/axis type.</summary>
		/// <param name="type">Module/axis type.</param>
		/// <param name="flags">Optionally one of Mcdlg.NameOnly or Mcdlg.DescOnly.</param>
		/// <param name="buffer">String buffer that will hold the descriptive string.</param>
		/// <param name="size">Size of buffer.</param>
		/// <returns>Pointer to the string buffer.</returns>
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_ModuleDescEx", CharSet = CharSet.Ansi)]
		public static extern IntPtr ModuleDescEx(Mcapi.ModuleType type, Flags flags, StringBuilder buffer, Int32 size);
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_RestoreAxis", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_RestoreAxis(Int16 hCtlr, Int16 axis, Flags flags, String privateinifile);
		/// <summary>Restore the settings of the given axis to a previously saved state.</summary>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="axis">Axis number.</param>
		/// <param name="flags">Optionally one or more of Mcdlg.CheckActive, Mcdlg.NoMotion, 
		/// Mcdlg.NoFilter, Mcdlg.NoPhase, Mcdlg.NoPosition, Mcdlg.NoScale, Mcdlg.NoHardLimits,
		/// Mcdlg.NoSoftLimits, Mcdlg.NoProfile, Mcdlg.NoMisc, or Mcdlg.Prompt, OR'ed together.</param>
		/// <param name="privateinifile">Name and optionally path of INI file to use.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error RestoreAxis(Mcapi controller, Int16 axis, Flags flags, String privateinifile)
		{
			return ProcessException(MCDLG_RestoreAxis(controller.Handle, axis, flags, privateinifile));
		}
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_RestoreDigitalIO", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_RestoreDigitalIO(Int16 hCtlr, Int16 startchannel, Int16 endchannel, String privateinifile);
		/// <summary>Restore the settings of a range of digital I/O channels.</summary>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="startchannel">Starting cahnnel number.</param>
		/// <param name="endchannel">Ending channel number.</param>
		/// <param name="privateinifile">Name and optionally path of INI file to use.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error RestoreDigitalIO(Mcapi controller, Int16 startchannel, Int16 endchannel, String privateinifile)
		{
			return ProcessException(MCDLG_RestoreDigitalIO(controller.Handle, startchannel, endchannel, privateinifile));
		}
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_SaveAxis", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_SaveAxis(Int16 hCtlr, Int16 axis, Flags flags, String privateinifile);
		/// <summary>Save the settings of the given axis to an initialization file for later use.</summary>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="axis">Axis number.</param>
		/// <param name="flags">Optionally one or more of Mcdlg.CheckActive, Mcdlg.NoMotion, 
		/// Mcdlg.NoFilter, Mcdlg.NoPhase, Mcdlg.NoPosition, Mcdlg.NoScale, Mcdlg.NoHardLimits,
		/// Mcdlg.NoSoftLimits, Mcdlg.NoProfile, or Mcdlg.NoMisc, OR'ed together.</param>
		/// <param name="privateinifile">Name and optionally path of INI file to use.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error SaveAxis(Mcapi controller, Int16 axis, Flags flags, String privateinifile)
		{
			return ProcessException(MCDLG_SaveAxis(controller.Handle, axis, flags, privateinifile));
		}
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_SaveDigitalIO", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_SaveDigitalIO(Int16 hCtlr, Int16 startchannel, Int16 endchannel, String privateinifile);
		/// <summary>Save the settings of a range of digital I/O channels.</summary>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="startchannel">Starting cahnnel number.</param>
		/// <param name="endchannel">Ending channel number.</param>
		/// <param name="privateinifile">Name and optionally path of INI file to use.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error SaveDigitalIO(Mcapi controller, Int16 startchannel, Int16 endchannel, String privateinifile)
		{
			return ProcessException(MCDLG_SaveDigitalIO(controller.Handle, startchannel, endchannel, privateinifile));
		}
		
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_Scaling", CharSet = CharSet.Ansi)]
		private static extern Int32 MCDLG_Scaling(IntPtr hwnd, Int16 hCtlr, Int16 axis, Flags flags, String title);
		/// <summary>Display a scaling setup dialog and, if the motion controller supports 
		/// scaling changes, allow the user to change the scaling parameters. </summary>
		/// <param name="hwnd">Parent window or NULL.</param>
		/// <param name="controller">Motion controller object.</param>
		/// <param name="axis">Axis number.</param>
		/// <param name="flags">Optionally one or more of Mcdlg.CheckActive and Mcdlg.Prompt, OR'ed together.</param>
		/// <param name="title">An optional title string for the dialog box. If this pointer is 
		/// NULL or points to a zero length string the default title is used.</param>
		/// <returns>An Mcapi.Error error status.</returns>
		public static Mcapi.Error Scaling(IntPtr hwnd, Mcapi controller, Int16 axis, Flags flags, String title)
		{
			return ProcessException(MCDLG_Scaling(hwnd, controller.Handle, axis, flags, title));
		}
		
		/// <summary>Select a motion controller from the installed controllers.</summary>
		/// <param name="hwnd">Handle to parent window or NULL.</param>
		/// <param name="CurrentID">ID of the motion controller currently in use.</param>
		/// <param name="flags">Reserved for future use (set to zero).</param>
		/// <param name="title">An optional title string for the dialog box. If this pointer is 
		/// NULL or points to a zero length string the default title is used.</param>
		/// <returns>New controller ID number.</returns>
		[DllImport(mcdlgdll, EntryPoint = "MCDLG_SelectController", CharSet = CharSet.Ansi)]
		public static extern Int16 SelectController(IntPtr hwnd, Int16 CurrentID, Flags flags, String title);

		//
		// -- Helper Function declarations -------------------------------------
		//
		private static Mcapi.Error ProcessException(Mcapi.Error error)
		{
			// TODO: throw exception if enabled
			return error;
		}

		private static Mcapi.Error ProcessException(Int32 error)
		{
			return ProcessException((Mcapi.Error)error);
		}
	}
}
